<?php

// Nastaví cestu ke knihovnám
set_include_path(get_include_path().PATH_SEPARATOR.__DIR__.'/library');

/**
 * Třída zavaděče aplikace
 *
 * @package    application
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Bootstrap extends Zend_Application_Bootstrap_Bootstrap {

  /**
   * Přetížení: zachytí volání __handleErrors nebo metod bootstrap<resourcename>()
   *
   * @param  string $method
   * @param  array  $args
   * @return void
   * @throws Zend_Application_Bootstrap_Exception  Na neplatný název metody
   */
  public function __call($method, $args) {
    if ('__handleErrors' == $method) {
      return call_user_func_array(array($this, $method), $args);
    }
    return parent::__call($method, $args);
  }

  /**
   * Obsluha fatální chyby
   *
   * @param Exception | Array $e
   * $param boolean $notify
   * @return void
   */
  protected function __handleErrors($e = null, $email = false) {
    header('HTTP/1.1 500 Internal Server Error', true);

    $text = false;
    $view = new Zend_View();
    $view->addScriptPath(__DIR__.'/system/views/scripts/');
    echo $view->assign('error', $e)->render('error/fatal.phtml');

    if ($email) {
      if (is_array($e)) {
        $text = 'Soubor: '.$e['file']."\n"
              . 'Zpráva: '.$e['message']."\n"
              . 'Řádek: '.$e['line']."\n"
              . 'URL: '.$_SERVER['REQUEST_URI']."\n";
      }
      if ($e instanceof Exception) {
        $text = 'Soubor: '.$e->getFile()."\n"
              . 'Zpráva: '.$e->getMessage()."\n"
              . 'Řádek: '.$e->getLine()."\n"
              . 'Stack: '.$e->getTraceAsString()."\n"
              . 'URL: '.$_SERVER['REQUEST_URI']."\n";
      }
    }

    if ($text) {
      $application = System_Model_Settings::getValue('application_title', 'Webbia Standard');
      try {
        $mail = new Zend_Mail('UTF-8');
        $mail->setSubject('Fatální chyba v aplikaci '.$application);
        $mail->addTo($email);
        $mail->setBodyText($text);
        $mail->send();
      } catch (Exception $e) {
        $messenger = new Zend_Controller_Action_Helper_FlashMessenger();
        $messenger->addMessage($e->getMessage());
      }
    }

    exit;
  }

  /**
   * Elegantní reakce na chybu při běhu aplikace
   *
   * @throws Exception
   */
  public function run() {
    $errorHandling = $this->getOption('errorhandling');
    try {
      parent::run();
    } catch (Exception $e) {
      if ((bool) $errorHandling['graceful']) {
        $this->__handleErrors($e, $errorHandling['email']);
      } else {
        throw $e;
      }
    }
  }

  /**
   * Registrace funkce pro obsluhu fatální chyby
   *
   * @return void
   */
  protected function _initShutDown() {
    $self = clone $this;
    $callback = function() use ($self) {
      $last = error_get_last();
      if (is_array($last) && !empty($last) && (E_ERROR == $last['type'])) {
        $errorHandling = $self->getOption('errorhandling');
        $self->__handleErrors($last, $errorHandling['email']);
      }
    };
    register_shutdown_function($callback);
  }

  /**
   * Zavedení autoloadingu
   *
   * @return Zend_Loader_Autoloader
   */
  protected function _initAutoload() {
    $systemPath = __DIR__.'/system';
    $options = array(
      'basePath'  => $systemPath,
      'namespace' => ucfirst(basename($systemPath)).'_'
    );

    new Zend_Application_Module_Autoloader($options);
    return Zend_Loader_Autoloader::getInstance();
  }

  /**
   * Zavedení databázového připojení
   *
   * @return Zend_Db
   */
  protected function _initDatabase() {
    $this->bootstrap('db');
    $db = $this->getResource('db');
    $config = $db->getConfig();

    try {
      $db->fetchOne('SELECT 1 AS dummy');
    } catch (Exception $e) {
      $path = THEME_PATH.'/logs/error-database.log';
      file_put_contents($path, $e->getMessage()."\n", FILE_APPEND);

      throw new Exception('Nepodařilo se spojit s databází.');
    }

    Zend_Registry::set('Zend_Db', $db);

    if ($config['profile']) {
      $profiler = new Zend_Db_Profiler_Firebug('Výpis databázových dotazů');
      $profiler->setEnabled(true);
      $db->setProfiler($profiler);
    }

    return $db;
  }

  /**
   * Inicializace cache
   *
   * @return Zend_Cache
   */
  protected function _initCaching() {
    $op = $this->getOption('cache');
    $cache = Zend_Cache::factory($op['frontend'], $op['backend'], $op['frontendoptions'], $op['backendoptions']);

    Zend_Registry::set('Zend_Cache', $cache);
    Zend_Db_Table_Abstract::setDefaultMetadataCache($cache);

    $customCache = new Custom_Cache();
    Zend_Registry::set('Custom_Cache', $customCache);

    return $cache;
  }

  /**
   * Zavedení lokálního nastavení
   *
   * @return Zend_Locale
   */
  protected function _initLocale() {
    $strLocale = Custom_Tools::getCurrentLocale();
    setlocale(LC_ALL, "{$strLocale}.utf8");

    $locale = new Zend_Locale($strLocale);
    Zend_Registry::set('Zend_Locale', $locale);

    return $locale;
  }

  /**
   * Nastavení logování
   *
   * @return Zend_Log
   */
  public function _initLogging() {
    $writer = new Zend_Log_Writer_Stream(THEME_PATH.'/logs/application.log');
    $logger = new Zend_Log();

    $logger->addWriter($writer);
    Zend_Registry::set('Zend_Log', $logger);

    return $logger;
  }

  /**
   * Zavedení nastavení view
   *
   * @return Zend_View
   */
  protected function _initViewSettings() {
    $this->bootstrap('view');
    $view = $this->getResource('view');

    $view->doctype('XHTML5');
    $view->application_title = System_Model_Settings::getValue('application_title', 'Webbia');
    $view->application_layout = System_Model_Settings::getValue('application_layout', '1-1-1');
    $view->languages = System_Model_Settings::getValue('languages', array('cs' => 'čeština'));

    $db = Zend_Registry::get('Zend_Db');
    $config = $db->getConfig();

    $view->application_profile = $config['profile'];
    $view->addHelperPath('Custom/View/Helper/', 'Custom_View_Helper_');

    $view->addScriptPath(__DIR__.'/system/views/scripts/');
    $view->addScriptPath(__DIR__.'/layouts/');
    $view->addScriptPath(THEME_PATH.'/scripts/');

    Zend_View_Helper_PaginationControl::setDefaultViewPartial('controls.phtml');

    Zend_Registry::set('Zend_View', $view);
    Zend_Registry::set('session', $this->getOption('session'));

    $view->language = Zend_Registry::get('Zend_Locale')->getLanguage();

    return $view;
  }

  /**
   * Zavedení nastavení překladů
   *
   * @todo Dodělat
   * @return Zend_Translate
   */
  protected function _initTranslating() {
    $this->bootstrap('translate');
    $translate = $this->getResource('translate');

    Zend_Registry::set('Zend_Translate', $translate);

    Zend_Form::setDefaultTranslator($translate);
    Zend_Validate_Abstract::setDefaultTranslator($translate);

    return $translate;
  }

  /**
   * Inicializace front controlleru
   *
   * @return Zend_Controller_Front
   */
  protected function _initFront() {
    $this->bootstrap('frontController');

    $front = $this->frontController;
    $front->addControllerDirectory(__DIR__.'/system/controllers');

    return $front;
  }

  /**
   * Zavedení směrování
   *
   * @return Zend_Controller_Router
   */
  protected function _initRouting() {
    $router = $this->frontController->getRouter();

    $parsedRequest = (object) Custom_Tools::parseRequest($_SERVER['REQUEST_URI']);
    if ('sitemap.xml' == $parsedRequest->file) {
      $sitemapAttrs = array('controller' => 'sitemap', 'action' => 'index');
      $sitemap = new Zend_Controller_Router_Route('sitemap.xml', $sitemapAttrs);
      $router->addRoute('sitemap', $sitemap);
    }
    if ('robots.txt' == $parsedRequest->file) {
      $robotsAttrs = array('controller' => 'sitemap', 'action' => 'robots');
      $robots = new Zend_Controller_Router_Route('robots.txt', $robotsAttrs);
      $router->addRoute('robots', $robots);
    }
    if (!$parsedRequest->is_index && $parsedRequest->file &&
        is_object($rewrite = System_Model_Rewrites::get($parsedRequest->file))) {
      $name = str_replace('-', '_', $rewrite->rewrite);
      $attrs = array(
        'controller' => $rewrite->controller,
        'action'     => $rewrite->action,
        'page'       => $parsedRequest->page
      );
      $map = array(1 => 'rewrite', 2 => 'page');
      $route = ".*?({$rewrite->rewrite})/?(\d+)?";

      $$name = new Zend_Controller_Router_Route_Regex($route, $attrs, $map, '%s/%s');
      $router->addRoute($name, $$name);
    }

    return $router;
  }

  /**
   * Inicializace routy pro dynamický obsah
   *
   * @return Zend_Controller_Router
   */
  protected function _initDynamicRoute() {
    $router = $this->frontController->getRouter();

    $route = '.*?'.System_Model_Dynamic::MATCH_EXPRESSION;
    $attrs = array('controller' => 'index', 'action' => 'dynamic');
    $map = array(1 => 'content');
    $pattern = '%s';

    $dynamic = new Zend_Controller_Router_Route_Regex($route, $attrs, $map, $pattern);
    $router->addRoute('dynamic', $dynamic);

    return $router;
  }

  /**
   * Přidá handlery do observeru pro indexaci
   *
   * @todo Dodělat
   * @return void
   */
  protected function _initSearch() {
    System_Model_Article::attachObserver('System_Model_Search', 'record');
    System_Model_Object::attachObserver('System_Model_Search', 'record');
  }

  /**
   * Zkusí přihlásit uživalete
   *
   * @return void
   */
  public function _initAutolog() {
    System_Model_Logins::autolog();
  }

}
