<?php

/**
 * Jednoduchá paměťová cache
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Custom_Cache {

  /**
   * Cache vypnuta
   *
   * @var boolean
   */
  protected $_disabled = false;

  /**
   * Uložená data do cache
   *
   * @var array
   */
  protected $_cache = array();

  /**
   * Uložený čas posledních změn
   *
   * @var array
   */
  protected $_mtime = array();

  /**
   * Konstruktor
   *
   * @return void
   */
  public function __construct() {
    $this->_cache = array();
    $this->_mtime = array();
  }

  /**
   * Otestuje, zdali je v cachi k dispozici daný identifikátor a pokud ano,
   * vrátí přiřazenou hodnotu; jinak vrací FALSE
   *
   * @param string $id  Identifikátor dat
   * @param boolean $doNotTestCacheValidity  Nepoužíváno
   * @param boolean $doNotUnserialize  Nepoužíváno
   * @return mixed|false  Nacachovaná data
   */
  public function load($id, $doNotTestCacheValidity = false, $doNotUnserialize = false) {
    if ($this->_disabled) {
      return false;
    }
    return array_key_exists($id, $this->_cache) ? $this->_cache[$id] : false;
  }

  /**
   * Otestuje čas poslední změny hodnoty daného identifikátoru
   *
   * @param string $id  Identifikátor dat
   * @return integer|false  Poslední změna nacachované hodnoty; jinak FALSE
   */
  public function test($id) {
    if ($this->_disabled) {
      return false;
    }
    return array_key_exists($id, $this->_mtime) ? $this->_mtime[$id] : false;
  }

  /**
   * Uloží data do cache
   *
   * @param mixed $data  Data vkládaná do cache
   * @param string $id  Identifikátor dat
   * @return boolean  TRUE, pokud se nevyskytne problém
   */
  public function save($data, $id) {
    if (array_key_exists($id, $this->_cache) && ($data === null)) {
      unset($this->_cache[$id], $this->_mtime[$id]);
    } else {
      $this->_cache[$id] = $data;
      $this->_mtime[$id] = time();
    }
    return true;
  }

  /**
   * Vyjme identifikátor dat z cache
   *
   * @param string $id  Identifikátor dat, který bude zrušen
   * @return boolean  TRUE, pokud se nevyskytne problém
   */
  public function remove($id) {
    $this->save(null, $id);
  }

  /**
   * Vyčistí kompletně paměťovou cache
   *
   * @param string $mode  Nepoužíváno
   * @param array|string $tags  Nepoužíváno
   * @return boolean  TRUE, pokud se nevyskytne problém
   */
  public function clean($mode = 'all', $tags = array()) {
    $result = true;

    $ids = $this->getIds();
    foreach ($ids as $id) {
      $result = $result && $this->remove($id);
    }

    return $result;
  }

  /**
   * Vrátí pole identifikátorů uložených v cachi
   *
   * @return array  Pole identifikátorů typu string
   */
  public function getIds() {
    if ($this->_disabled) {
      return array();
    }
    return array_keys($this->_cache);
  }

  /**
   * Zaktualizuje čas vložení
   *
   * @param string $id  Identifikátor dat
   * @param integer $extraLifetime  Nepoužíváno
   * @return boolean  TRUE, pokud se nevyskytne problém
   */
  public function touch($id, $extraLifetime) {
    if (array_key_exists($id, $this->_mtime)) {
      $this->_mtime[$id] = time();
      return true;
    }
    return false;
  }

  /**
   * Zakáže nebo povolí používání cache
   *
   * @param boolean $flag
   */
  public function setDisabled($flag = true) {
    $this->_disabled = (bool) $flag;
  }

  /**
   * Otestuje stav cache - je-li zakázána nebo povolena
   *
   * @return boolean
   */
  public function isDisabled() {
    return $this->_disabled;
  }

}
