<?php

/**
 * Třída pro práci s řádkem databázové tabulky rozšířený o události nad řádkem
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Custom_Db_Table_Row_Observer extends Zend_Db_Table_Row_Abstract {

  const OBSERVE_ALL         = 'eventAll';

  const OBSERVE_PRE_INSERT  = 'eventPreInsert';
  const OBSERVE_POST_INSERT = 'eventPostInsert';

  const OBSERVE_PRE_UPDATE  = 'eventPreUpdate';
  const OBSERVE_POST_UPDATE = 'eventPostUpdate';

  const OBSERVE_PRE_DELETE  = 'eventPreDelete';
  const OBSERVE_POST_DELETE = 'eventPostDelete';

  /**
   * Pole observerů
   *
   * @var array
   */
  protected static $_observers = array();

  /**
   * Napojí třídu $class a metodu $method na událost $event
   *
   * @param string $class
   * @param string $method
   * @param string $event
   * @return boolean
   */
  public static function attachObserver($class, $method, $event = self::OBSERVE_ALL) {
    if (!is_string($class) || !is_string($method) || !class_exists($class) ||
        !is_callable(array($class, $method))) {
      return false;
    }

    self::$_observers[$class][$method] = $event;

    return true;
  }

  /**
   * Odpojí třídu $class a metodu $method od událostí
   *
   * @param string $class
   * @param string $method
   * @return boolean
   */
  public static function detachObserver($class, $method = null) {
    if (!is_string($class) || !isset(self::$_observers[$class])) {
      return false;
    }

    if (is_string($method) && isset(self::$_observers[$class][$method])) {
      unset(self::$_observers[$class][$method]);
    } else {
      unset(self::$_observers[$class]);
    }

    return true;
  }

  /**
   * Metoda pro volání napojených tříd a metod
   *
   * @param string $event
   * @return void
   */
  protected function _notifyObservers($event) {
    if (!empty(self::$_observers)) {
      foreach (self::$_observers as $obsClass => $obsMethods) {
        foreach ($obsMethods as $obsMethod => $obsEvent) {
          if (($event == $obsEvent) || ($obsEvent == self::OBSERVE_ALL)) {
            call_user_func(array($obsClass, $obsMethod), $event, $this);
          }
        }
      }
    }
  }

  /**
   * Vloží řádek
   *
   * @return mixed  Hodnota primárního(ch) klíče(ů)
   */
  protected function _doInsert() {
    $this->_notifyObservers(self::OBSERVE_PRE_INSERT);
    $result = parent::_doInsert();
    $this->_notifyObservers(self::OBSERVE_POST_INSERT);
    return $result;
  }

  /**
   * Aktualizuje řádek
   *
   * @return mixed  Hodnota primárního(ch) klíče(ů)
   */
  protected function _doUpdate() {
    $this->_notifyObservers(self::OBSERVE_PRE_UPDATE);
    $result = parent::_doUpdate();
    $this->_notifyObservers(self::OBSERVE_POST_UPDATE);
    return $result;
  }

  /**
   * Smaže rádek
   *
   * @return integer  Počet smazaných řádků
   */
  public function delete() {
    $this->_notifyObservers(self::OBSERVE_PRE_DELETE);
    $result = parent::delete();
    $this->_notifyObservers(self::OBSERVE_POST_DELETE);
    return $result;
  }

}
