<?php

/**
 * Třída pro práci s řádkem databázové tabulky a atribut rewrite
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
abstract class Custom_Db_Table_Row_Rewrite extends Custom_Db_Table_Row_Observer {

  const REWRITE_PRIMARY_KEY = 'rewrite';

  /**
   * „Surová“ data z formuláře
   *
   * @var array
   */
  protected $_rawData = array();

  /**
   * Instance třídy modelu pro rewrity
   *
   * @var System_Model_Rewrites
   */
  protected $_modelRewrites = null;

  /**
   * Vytvořený objekt rewritu
   *
   * @var System_Model_Rewrite
   */
  protected $_objectRewrite = null;

  /**
   * Jazyková mutace rewritu
   *
   * @var string
   */
  protected $_language = null;

  /**
   * Rewrite
   *
   * @var string
   */
  protected $_rewrite = null;

  /**
   * Konstruktor
   *
   * @param  array $config  VOLITELNÝ uživatelem definované pole konfigurace
   * @return void
   * @throws Zend_Db_Table_Row_Exception
   */
  public function __construct(array $config = array()) {
    parent::__construct($config);
    $this->_modelRewrites = $this->getTable()->getRewritesModel();
  }

  /**
   * Nastaví všechna data z ​​pole
   *
   * @param  array $data
   * @return Zend_Db_Table_Row_Abstract Poskytuje fluent rozhraní
   */
  public function setFromArray(array $data) {
    $this->_rawData = $data;
    return parent::setFromArray($data);
  }

  /**
   * Umožní aplikovat změnu řádku před vložením.
   *
   * @return void
   */
  protected function _insert() {
    $this->_beforeSave();

    parent::_insert();
  }

  /**
   * Umožní aplikovat změnu řádku po vložení.
   *
   * @return void
   */
  protected function _postInsert() {
    parent::_postInsert();

    $this->_afterSave();
  }

  /**
   * Umožní aplikovat změnu řádku před uložením.
   *
   * @return void
   */
  protected function _update() {
    $this->_beforeSave();

    parent::_update();
  }

  /**
   * Umožní aplikovat změnu řádku po uložení.
   *
   * @return void
   */
  protected function _postUpdate() {
    parent::_postUpdate();

    $this->_afterSave();
  }

  /**
   * Umožní aplikovat změnu řádku po smazání.
   *
   * @return void
   */
  protected function _postDelete() {
    parent::_postDelete();

    $this->_deleteRewrite();
  }

  /**
   * Vrátí titulek objektu
   *
   * @return string
   */
  protected function _getCaption() {
    $caption = self::REWRITE_PRIMARY_KEY;
    if (array_key_exists('caption', $this->_rawData)) {
      $caption = $this->_rawData['caption'];
    }
    if (!empty($this->_rawData[self::REWRITE_PRIMARY_KEY])) {
      $caption = $this->_rawData[self::REWRITE_PRIMARY_KEY];
    }
    return $caption;
  }

  /**
   * Vytvoří nový rewrite
   *
   * @return boolean  TRUE v případě, že byl rewrite vytvořen nebo změněn; jinak FALSE
   */
  protected function _makeRewrite() {
    $isNew = empty($this->_cleanData);
    $isRewriteable = (array_key_exists(self::REWRITE_PRIMARY_KEY, $this->_cleanData) ||
                      array_key_exists(self::REWRITE_PRIMARY_KEY, $this->_rawData) ||
                      array_key_exists(self::REWRITE_PRIMARY_KEY, $this->_data));

    $cleanRewrite = isset($this->_cleanData[self::REWRITE_PRIMARY_KEY]) ? $this->_cleanData[self::REWRITE_PRIMARY_KEY] : '';
    $dataRewrite = isset($this->_data[self::REWRITE_PRIMARY_KEY]) ? $this->_data[self::REWRITE_PRIMARY_KEY] : '';

    if ($isRewriteable) {
      $modified = ($isNew || ($cleanRewrite != $dataRewrite));

      if ($modified && !$isNew) {
        $this->_deleteRewrite();
      }

      if ($modified) {
        $caption = $this->_getCaption();
        $language = isset($this->language) ? $this->language : '';
        $this->rewrite = $this->_modelRewrites->getRewrite($caption, $language);
      }
    }

    return isset($this->rewrite) && ($cleanRewrite != $this->rewrite);
  }

  /**
   * Uloží rewrite do databáze
   *
   * @return mixed  Primární klíč nebo FALSE
   */
  protected function _saveRewrite() {
    $result = false;

    $language = isset($this->language) ? $this->language : '';
    if (!is_object($this->_objectRewrite = $this->_modelRewrites->getRewriteById($this->_data['rewrite'], $language))) {
      $this->_objectRewrite = $this->_modelRewrites->createRow();
    }

    if (isset($this->_rawData['_controller']) && isset($this->_rawData['_action'])) {
      $parts = explode('_', get_class($this));
      $object = strtolower(end($parts));

      $this->_objectRewrite->rewrite = $this->rewrite;
      $this->_objectRewrite->controller = $this->_rawData['_controller'];
      $this->_objectRewrite->action = $this->_rawData['_action'];
      $this->_objectRewrite->object = $object;
      $this->_objectRewrite->language = $language;
      $result = $this->_objectRewrite->save();
    }

    return $result;
  }

  /**
   * Rutiny před vložením nebo uložením
   *
   * @return void
   */
  private function _beforeSave() {
    if ($this->_makeRewrite()) {
      $this->_saveRewrite();
    }
  }

  /**
   * Rutiny po vložení nebo uložení
   *
   * @return void
   */
  private function _afterSave() {
    $primary = $this->_getPrimaryKey();
    if (!$primary) {
      $this->_deleteRewrite();
    }
  }

  /**
   * Nízkoúrovňové smazání rewritu
   *
   * @return integer
   */
  private function _deleteRewrite() {
    $where = array(
      'rewrite LIKE ? '  => $this->_cleanData['rewrite'],
      'language LIKE ? ' => $this->_cleanData['language']
    );
    $table = $this->_modelRewrites->info('name');
    return $this->getTable()->getAdapter()->delete($table, $where);
  }

  /**
   * Vrátí url objektu
   *
   * @return string
   */
  abstract public function getUrl();

}
