<?php

/**
 * Třída upraveného základu formuláře s rozšířením o desetinná čísla
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class Custom_Form extends Zend_Form {

  /**
   * Elementy, které obsahují desetinné hodnoty
   *
   * @var array
   */
  protected $_floats = array();

  /**
   * Locale
   *
   * @var Zend_Locale
   */
  protected $_locale = null;

  /**
   * Konstruktor
   *
   * Nastaví rozšiřující vlastnosti
   *
   * @param mixed $options
   * @return void
   */
  public function __construct($options = null) {
    $this->_beforeInit();
    parent::__construct($options);
  }

  /**
   * Přidá element, který může obsahovat desetinné číslo
   *
   * @param  string|Zend_Form_Element $element
   * @param  string $name
   * @param  array|Zend_Config $options
   * @throws Zend_Form_Exception on invalid element
   * @return Custom_Form
   */
  public function addFloatElement($element, $name = null, $options = null) {
    parent::addElement($element, $name, $options);
    if ($element instanceof Zend_Form_Element) {
      $name = $element->getName();
    }
    $this->_floats[] = $name;
    return $this;
  }

  /**
   * Nastaví objekt locale
   *
   * @param  Zend_Locale $locale
   * @return Custom_Form
   */
  public function setLocale(Zend_Locale $locale = null) {
    $this->_locale = $locale;
    return $this;
  }

  /**
   * Vrátí objekt locale
   *
   * Pokud není žádný zaregistrován, pokusí se jej převzít ze Zend_Registry
   *
   * @return Zend_Locale|null
   */
  public function getLocale() {
    if (null === $this->_locale) {
      require_once 'Zend/Registry.php';
      if (Zend_Registry::isRegistered('Zend_Locale')) {
        $locale = Zend_Registry::get('Zend_Locale');
        if ($locale instanceof Zend_Locale) {
          return $locale;
        }
      }
    }
    return $this->_locale;
  }

  /**
   * Nastaví výchozí hodnoty elementů
   *
   * Nastaví hodnoty pro všechny elementy uvedené v poli $defaults.
   *
   * @param  array $defaults
   * @return Custom_Form
   */
  public function setDefaults(array $defaults) {
    $data = $this->_convertFloatNumber($defaults, 'toFloat');
    return parent::setDefaults($data);
  }

  /**
   * Načte všechny hodnoty elementů formuláře
   *
   * @param  boolean $suppressArrayNotation
   * @return array
   */
  public function getValues($suppressArrayNotation = false) {
    $data = parent::getValues($suppressArrayNotation);
    return $this->_convertFloatNumber($data, 'getFloat');
  }

  /**
   * Vrátí pouze platné hodnoty z daného formulářového vstupu.
   *
   * @param array $data
   * @param boolean $suppressArrayNotation
   * @return array
   */
  public function getValidValues($data, $suppressArrayNotation = false) {
    $values = parent::getValidValues($data, $suppressArrayNotation);
    return $this->_convertFloatNumber($values, 'getFloat');
  }

  /**
   * Nastaví view, locale, translator a vymaže pole pro elementy s desetinnými čísly
   *
   * @return void
   */
  protected function _beforeInit() {
    $this->_floats = array();
    $this->setView($this->getView());
    $this->setLocale($this->getLocale());
    $this->setTranslator($this->getTranslator());
    $this->addPrefixPath('Custom_Form_Element_', 'Custom/Form/Element/', self::ELEMENT);
    $this->addElementPrefixPath('Custom_Validate', 'Custom/Validate/', 'Validate');
  }

  /**
   * Přetransformuje v poli $data hodnoty float na string nebo opačně
   *
   * @param array $data
   * @param string $method
   * @return array
   */
  protected function _convertFloatNumber($data, $method) {
    if (is_array($data) && count($this->_floats)) {
      $options = array('locale' => $this->_locale);
      foreach ($this->_floats as $elementName) {
        if (array_key_exists($elementName, $data) && isset($data[$elementName])) {
          if (Zend_Locale_Format::isNumber($data[$elementName], $options) && ('toFloat' == $method)) {
            continue;
          }
          $data[$elementName] = Zend_Locale_Format::$method($data[$elementName], $options);
        }
      }
    }
    return $data;
  }

}
