<?php

require_once 'Zend/Locale.php';
require_once 'Zend/Translate/Adapter.php';

/**
 * Třída adaptéru překladače pro data z databáze
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class Custom_Translate_Adapter_Dtb extends Zend_Translate_Adapter {

  /**
   * Databázová tabulka pro překlady
   *
   * @var System_Model_Texts
   */
  protected $_modelTexts = null;

  /**
   * Načte překladová dat
   *
   * @param  mixed   $table     Model databázové tabulky
   * @param  string  $locale    Locale dat, identický s identifikátorem národní prostředí
   * @param  array   $options   OPTIONAL Možnosti použití
   * @throws Zend_Translate_Exception
   * @return void
   */
  protected function _loadTranslationData($table, $locale, array $options = array()) {
    if (is_string($table)) {
      require_once 'Zend/Loader/Autoloader.php';
      Zend_Loader_Autoloader::autoload($table);
      $this->_modelTexts = new $table();
    }

    if (!$this->_modelTexts instanceof Zend_Db_Table_Abstract) { /* @var $table System_Model_Texts */
      require_once 'Zend/Translate/Exception.php';
      throw new Zend_Translate_Exception("Databázová tabulka musí být instancí „Zend_Db_Table_Abstract“.");
    }

    return $this->_loadTexts();
  }

  /**
   * Překloží daný řetězec a vrátí překlad
   *
   * @see Zend_Locale
   * @param  string|array       $messageId Překládaný řetězec nebo Array pro plurálové překlady
   * @param  string|Zend_Locale $locale    OPTIONAL Locale/Jazyk pro nastavení, identický
   *                                       s identifikátorem národní prostředí,
   *                                       @see Zend_Locale pro více informací
   * @return string
   */
  public function translate($messageId, $locale = null) {
    if (empty($messageId)) {
      return '';
    }

    if ($locale === null) {
      $locale = $this->_options['locale'];
    }

    $zendLocale = ($locale instanceof Zend_Locale) ? clone $locale : new Zend_Locale($locale);
    $language = $zendLocale->getLanguage();

    if (!array_key_exists($messageId, $this->_translate[$language])) {
      Zend_Registry::get('Zend_Log')->warn("ID '{$messageId}' nepřeloženo");
      $messageId = "<em style=\"color: red; font-weight: bold;\" title=\"Nepřeloženo\">{$messageId}</em>";
    }

    return parent::translate($messageId, $locale);
  }

  /**
   * Vrátí seznam jazyků
   *
   * @return array
   */
  public function getList() {
    $cols = $this->_modelTexts->info(Zend_Db_Table_Abstract::COLS);
    $primary = $this->_modelTexts->info(Zend_Db_Table_Abstract::PRIMARY);
    return array_diff($cols, $primary);
  }

  /**
   * Přidá překlad do databáze
   *
   * @param string $messageId
   * @param string $content
   * @param string $language
   * @return mixed Primární klíč(e), asociativní pole v případě složeného klíče,
   *     skalár v případě jednoho sloupce.
   */
  public function add($messageId, $content, $language) {
    if (!is_object($text = $this->_modelTexts->getId($messageId))) {
      $text = $this->_modelTexts->addId($messageId);
    }
    $text->{$language} = $content;
    $result = $text->save();

    if ($result) {
      $this->_translate[$language][$text->id] = $text->{$language};
    }

    return $result;
  }

  /**
   * Odebere překlad z databáze
   *
   * @param string $messageId
   * @param string $language
   * @return integer  Počet smazaných záznamů
   */
  public function remove($messageId, $language = null) {
    if ($this->_modelTexts->hasId($messageId)) {
      $result = $this->_modelTexts->removeId($messageId);
      if ($result && isset($language) && isset($language, $this->_translate[$language])) {
        unset($this->_translate[$language][$messageId]);
      }
      return $result;
    }
    return (int) false;
  }

  /**
   * Načte veškeré překlady z databáze
   *
   * @return void
   */
  protected function _loadTexts() {
    $language = Zend_Registry::get('Zend_Locale')->getLanguage();
    return array(
      $language => $this->_modelTexts->getTranslates($language)
    );
  }

  /**
   * Vrátí název adaptéru
   *
   * @return string
   */
  public function toString() {
    return 'Dtb';
  }

}
