<?php

/**
 * Třída formuláře pro úpravu souboru
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Form_File extends System_Form_Abstract {

  const NEW_OBJECT_TEXT  = 'System_Objects_File_New';
  const EDIT_OBJECT_TEXT = 'System_Objects_File_Edit';

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/file.phtml')),
    'Form'
  );

  /**
   * Model pro objekty
   *
   * @var System_Model_Objects
   */
  protected $_modelObjects = null;

  /**
   * SESSION pro akce formuláře
   *
   * @var Zend_Session_Namespace
   */
  protected $_session = null;

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    parent::init();

    $this->setAttrib('id', 'file-form');
    $this->setEnctype(Zend_Form::ENCTYPE_MULTIPART);

    $this->_modelObjects = new System_Model_Objects();
    $folderOptions = array('0' => $this->_view->translate('System_Objects_Root_Folder'))
                   + $this->_padOptions($this->_modelObjects->getFolders()->getCaptions());
    $folderKeys = array_keys($folderOptions);

    $this->_session = new Zend_Session_Namespace(__CLASS__);
    if (!isset($this->_session->remove) ||
        Zend_Controller_Front::getInstance()->getRequest()->isGet()) {
      $this->_session->remove = array();
    }

    // - data -
    $this->getElement('language')->setLabel($this->_view->translate('System_Form_File_Language'));

    $id_parent = $this->createElement('select', 'id_parent');
    $id_parent->setDisableTranslator(true);
    $id_parent->setLabel($this->_view->translate('System_Form_File_Parent'));
    $id_parent->setMultiOptions($folderOptions);
    $id_parent->setValidators(array(
      array('InArray', true, array('haystack' => $folderKeys))
    ));
    $id_parent->setFilters(array('Int', 'Null'));
    $this->addElement($id_parent);

    if (is_object($caption = $this->getElement('caption'))) {
      $caption->setLabel('System_Form_File_Caption');
      $caption->setRequired(false);
    }

    $file = $this->createElement('file', 'file');
    $file->setLabel('System_Form_File_File');
    $file->setValidators(array(
      array('Extension', false, implode(',', array_keys(System_Model_FileType::$types)))
    ));
    $file->setDestination(System_Model_File::getUploadDir());
    $this->addElement($file);

    $existing = $this->createElement('text', 'existing');
    $existing->setLabel('System_Form_File_Existing');
    $existing->setRequired(false);
    $existing->setFilters(array('StringTrim', 'Null'));
    $existing->setDecorators(array('Label', 'ViewHelper', 'Errors'));
    $this->addElement($existing);

    $existbtn = $this->createElement('button', 'existbtn');
    $existbtn->setDisableTranslator(true);
    $existbtn->setLabel('...');
    $existbtn->setDecorators(array('ViewHelper'));
    $this->addElement($existbtn);

    if (is_object($is_visible = $this->getElement(System_Model_Object::ATTR_IS_VISIBLE))) {
      $is_visible->setLabel('System_Form_File_Is_Visible');
    }

    $this->getElement('description')->setLabel('System_Form_File_Description');
    $this->getElement('rewrite')->setLabel('System_Form_File_Rewrite');
    $this->getElement('object_type')->setValue(System_Model_Object::SYSTEM_OBJECT_FILE);
  }

  /**
   * Metoda pro zpracování dat formuláře.
   *
   * Metoda ověří data na platnost a provede určitou logiku nad daty
   *
   * @param array $values                Data formuláře
   * @param System_Model_Object $object  Vytvořený objekt
   * @return boolean                     Data ověřena a objekt uložen
   */
  public function process(array $values, System_Model_Object &$object = null) {
    if (isset($values['remove']) && is_array($values['remove']) &&
        is_numeric($id_file = key($values['remove'])) &&
        is_object($file = System_Model_Files::getFile($id_file))) {
      $this->_session->remove[] = $id_file;
      $this->setDefaults($values);
      return false;
    }

    if (is_bool($hasFile = $this->_hasFile($values)) && $hasFile) {
      $this->removeElement('file');
    }

    if (!$this->isValid($values)) {
      return false;
    }

    $data = $this->getValues();
    if (!$hasFile && empty($data['file']) && empty($data['existing'])) {
      $this->getElement('existing')->addError('System_Form_File_One_Must_Be_Filled');
      return false;
    }

    if (!is_object($object = $this->_modelObjects->getObjectById($data['id_object'], $data['language']))) {
      $object = $this->_modelObjects->createRow();
    }

    if (!empty($this->_session->remove)) {
      $id_file = array_pop($this->_session->remove);
      $objects = $this->_modelObjects->fetchAll(array('id_file = ?' => $id_file));
      if ((1 == $objects->count()) &&
          is_object($obj = $objects->current()) &&
          ($obj->id_object == $data['id_object']) &&
          ($obj->language == $data['language']) &&
          is_object($file = $obj->getFile())) {
        $file->delete();
      }
    }

    $file = null;
    if ($data['file']) {
      $fileInfo = !$hasFile ? $this->getElement('file')->getFileInfo() : array(array('error' => UPLOAD_ERR_NO_FILE));
      $fileData = reset($fileInfo);
      if (UPLOAD_ERR_OK == $fileData['error']) {
        $file = System_Model_File::saveFile($fileData['tmp_name']);
      }
    }
    if ($data['existing']) {
      $fileObject = $this->_modelObjects->getObjectByCode($data['existing']);
      $file = $fileObject->getFile();
    }
    if (is_object($file)) {
      $object->id_file = $file->id_file;
      if (empty($data['caption'])) {
        $data['caption'] = $file->getNameOnly();
      }
    }

    $object->object_type = System_Model_Object::SYSTEM_OBJECT_FILE;
    if ($object->setFromArray($data)->save()) {
      return true;
    }

    return false;
  }

  /**
   * Nastaví výchozí hodnoty elementům
   *
   * Nastaví hodnoty všech elementů uvedených v poli $defaults.
   *
   * @param  array $defaults
   * @return System_Form_File
   */
  public function setDefaults(array $defaults) {
    parent::setDefaults($defaults);

    if (is_numeric($id_object = $this->getElement('id_object')->getValue()) &&
        is_string($language = $this->getElement('language')->getValue()) &&
        is_object($object = $this->_modelObjects->getObjectById($id_object, $language)) &&
        is_object($file = $object->getFile()) &&
        empty($this->_session->remove)) {
      $this->getDisplayGroup('submits')->setAttrib('file', $file);
    }

    return $this;
  }

  /**
   * Doplní mezery (&nbsp;) před položky
   *
   * @param array $options
   * @return array
   */
  private function _padOptions(array $options = array()) {
    $last = end($options);
    array_walk($options, function (&$option) use ($last) {
      $sign = ($option == $last) ? '└' : '├';
      $option = " {$sign} {$option}";
    });
    return $options;
  }

  /**
   * Otestuje, zdali existuje objekt se souborem a není ve stavu změny souboru
   *
   * @param array $values
   * @return boolean
   */
  private function _hasFile(array $values) {
    if (is_object($object = $this->_modelObjects->getObjectById($values['id_object'], $values['language'])) &&
        is_object($file = $object->getFile()) &&
        empty($this->_session->remove)) {
      return true;
    }
    return false;
  }

}
