<?php

/**
 * Třída pro práci s řádkem databázové tabulky webbia_articles
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Article extends Custom_Db_Table_Row_Rewrite
                           implements Custom_Search_Interface {

  const ATTR_IS_VISIBLE  = 'is_visible';
  const ATTR_IS_MENUITEM = 'is_menuitem';

  const PART_SHOW_TITLE  = 'show_title';
  const PART_SHOW_FOOTER = 'show_footer';

  const MAX_KEYWORD_COUNT      = 500;
  const MAX_DESCRIPTION_LENGTH = 150;

  /**
   * Možnosti nastavení článku
   *
   * @var array
   */
  public static $ATTR_OPTIONS = array(
    self::ATTR_IS_VISIBLE  => 'System_Article_Is_Published',
    self::ATTR_IS_MENUITEM => 'System_Article_Is_Menuitem'
  );

  /**
   * Možnosti zobrazení článku
   *
   * @var array
   */
  public static $PART_OPTIONS = array(
    self::PART_SHOW_TITLE  => 'System_Article_Display_Title',
    self::PART_SHOW_FOOTER => 'System_Article_Display_Footer'
  );

  /**
   * Přednastavené možnosti článku
   *
   * @var array
   */
  public static $PART_DEFAULTS = array(
    self::PART_SHOW_TITLE  => true,
    self::PART_SHOW_FOOTER => false
  );

  /**
   * Název třídy objektu Zend_Db_Table_Abstract
   *
   * @var string
   */
  protected $_tableClass = 'System_Model_Articles';

  /**
   * Paměťová cache
   *
   * @var Custom_Cache
   */
  protected $_memoryCache = null;

  /**
   * Regulární výraz pro rozdělení slov obsahu do klíčových slov
   *
   * @var string
   */
  private $_split = '/[\n\r\t\,\.\:\;\!\?\s]+|(\&nbsp;|\&ndash;)+/';

  /**
   * Pořadí nového článku
   *
   * @var integer
   */
  private $_newOrder = null;

  /**
   * Drobečky externí navigace
   *
   * @var array
   */
  private $_crumbs = array();

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    parent::init();

    if (Zend_Registry::isRegistered('Custom_Cache')) {
      $this->_memoryCache = Zend_Registry::get('Custom_Cache');
    }
  }

  /**
   * Nastaví všechna data objektu z pole
   *
   * @param  array $data
   * @return Zend_Db_Table_Row_Abstract  Poskytuje fluent zápis
   */
  public function setFromArray(array $data) {
    $tmp = array(
      '_controller' => 'article',
      '_action'     => 'index'
    );
    return parent::setFromArray($data + $tmp);
  }

  /**
   * Vrátí obsah s nahrazenými dynamickými obsahy
   *
   * @return string
   */
  public function getContent() {
    $key = md5(__METHOD__.$this->id_article.$this->language);
    if (!is_object($this->_memoryCache) || (false === ($result = $this->_memoryCache->load($key)))) {
      $content = System_Model_Dynamic::content($this, $this->_crumbs);
      $result = (object) array('content' => $content, 'crumbs' => $this->_crumbs);
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }
    $this->_crumbs = $result->crumbs;
    return $result->content;
  }

  /**
   * Vrátí klíčová slova článku jako řetězec, slova jsou oddělená čárkou
   *
   * @return string
   */
  public function getKeywords() {
    if (null !== $this->keywords) {
      return $this->keywords;
    }

    $content = $this->getContent();

    $captionWords = preg_split($this->_split, strip_tags($this->caption));
    $contentWords = preg_split($this->_split, strip_tags($content));
    $keywords = array_slice(
      array_unique(
        array_filter(
          array_merge($captionWords, $contentWords),
          function($word) {
            return strlen($word) > 2;
          }
        )
      ),
      0,
      self::MAX_KEYWORD_COUNT
    );

    return implode(', ', $keywords);
  }

  /**
   * Vrátí popis stránky
   *
   * @return string
   */
  public function getDescription() {
    $length = self::MAX_DESCRIPTION_LENGTH;
    $description = preg_replace(
      '/\s+?(\S+)?$/',
      '',
      substr(
        trim(
          preg_replace(
            '/\s+/',
            ' ',
            preg_replace(
              '/[\n\r\t\s– ]|\&nbsp;|\&ndash;/',
              ' ',
              strip_tags($this->getContent())
            )
          )
        ),
        0,
        $length + 1
      )
    );
    return substr($description, 0, $length);
  }

  /**
   * Vrátí drobečky navigace článku
   *
   * @return array  Asociativní pole hodnot drobečkové navigace
   */
  public function getBreadcrumbs() {
    $first = $this->getTable()->getFirstArticle();
    if ($this->id_article == $first->id_article) {
      return $this->_crumbs;
    }

    $front = Zend_Controller_Front::getInstance();
    $ctrl = $front->getRequest()->getControllerName();

    $crumb  = array('label' => $this->caption, 'title' => $this->caption);
    $crumb += in_array($ctrl, array('index', 'article'))
            ? array('uri' => $this->getUrl(), 'active' => true)
            : array(
                'controller' => $front->getRequest()->getControllerName(),
                'action'     => $front->getRequest()->getActionName(),
                'params'     => array('id' => $this->id_article),
                'active'     => true
              );

    return array_merge(array($crumb), $this->_crumbs);
  }

  /**
   * Vrátí URL článku
   *
   * @return string  URL článku
   */
  public function getUrl() {
    if (null !== $this->redirect) {
      return $this->redirect;
    }
    $url = $this->rewrite;
    if ('' != ($baseUrl = Zend_Controller_Front::getInstance()->getBaseUrl())) {
      $url = $baseUrl . '/' . $url;
    }
    return $url;
  }

  /**
   * Vrátí kód URL článku
   *
   * @return string  URL článku
   */
  public function getCode() {
    return "[[article id={$this->id_article}]]";
  }

  /**
   * Vrátí data pro zaindexování do Zend_Search_Lucene
   *
   * @return array  Asociativní pole s akcí a daty
   */
  public function getDocuments() {
    $this->_refresh();

    $id_document = get_class($this->getTable()).':'.$this->id_article.':'.$this->language;

    $htmlContent = Zend_Search_Lucene_Document_Html::loadHTML($this->getContent(), true, 'UTF-8');
    $content = $htmlContent->getField('body');
    $content->name = 'content';

    $document = new Zend_Search_Lucene_Document();
    $document->addField(Zend_Search_Lucene_Field::keyword('id_document', $id_document));
    $document->addField(Zend_Search_Lucene_Field::text('caption', $this->caption, 'UTF-8'));
    $document->addField($content);

    $key = ($this->is_visible)
         ? Custom_Search_Interface::INDEX_ACTION_UPDATE
         : Custom_Search_Interface::INDEX_ACTION_REMOVE;

    $documents = array(
      array($key => $document)
    );

    return $documents;
  }

  /**
   * Vrátí čitelnost článku
   *
   * @return boolean  TRUE v případě čitelnosti, jinak FALSE
   */
  public function isReadable() {
    $readable = true;
    if (!Zend_Auth::getInstance()->hasIdentity() && !$this->is_visible) {
      $readable = false;
    }
    return $readable;
  }

  /**
   * Umožní aplikovat změnu řádku před vložením.
   *
   * @return void
   */
  protected function _insert() {
    $auth = Zend_Auth::getInstance();
    $this->id_user_creator = $auth->hasIdentity() ? $auth->getIdentity()->id_user : 0;
    if (!is_numeric($this->id_article)) {
      $this->id_article = null;
    }
    if (null === $this->date_create) {
      $this->date_create = date('Y-m-d H:i:s');
    }
    $this->_newOrder = $this->_getOrder();

    parent::_insert();
  }

  /**
   * Umožní aplikovat změnu řádku po uložení.
   *
   * @return void
   */
  protected function _postInsert() {
    parent::_postInsert();

    $table = $this->getTable();
    if (null === $this->_newOrder) {
      $this->_newOrder = $this->id_article;
    }

    $sql = "UPDATE {$table->info('name')} SET article_order = "
         . "{$this->_newOrder} WHERE id_article = {$this->id_article}";
    $table->getAdapter()->query($sql);
  }

  /**
   * Umožní aplikovat změnu řádku před uložením.
   *
   * @return void
   */
  protected function _update() {
    $auth = Zend_Auth::getInstance();
    $this->id_user_changer = $auth->hasIdentity() ? $auth->getIdentity()->id_user : 0;
    $this->date_change = date('Y-m-d H:i:s');

    parent::_update();
  }

  /**
   * Vrátí pořadí článku
   *
   * @return integer
   */
  private function _getOrder() {
    if (is_numeric($this->id_article) &&
        is_object($article = $this->getTable()->getMutationById($this->id_article))) {
      return $article->article_order;
    }
    return null;
  }

}
