<?php

/**
 * Třída pro práci s tabulkou webbia_articles
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Articles extends Custom_Db_Table_Rewrite {

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_articles';

  /**
   * Sloupce primárního klíče
   *
   * @var array
   */
  protected $_primary = array('id_article', 'language');

  /**
   * Regionální nastavení
   *
   * @var Zend_Locale
   */
  protected $_locale = null;

  /**
   * Třída řádku
   *
   * @var string
   */
  protected $_rowClass = 'System_Model_Article';

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    parent::init();
    $this->_locale = Zend_Registry::get('Zend_Locale');
  }

  /**
   * Vrátí článek dle $id_article a $language
   *
   * @param integer $id_article  ID článku
   * @param string $language  Jazyk článku
   * @return System_Model_Article
   */
  public function getArticleById($id_article, $language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }

    $key = md5(__METHOD__.$id_article.$language);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $result = $this->find($id_article, $language)->current();
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí mutaci dle $id_article
   *
   * @param integer $id_article  ID článku
   * @return System_Model_Article
   */
  public function getMutationById($id_article) {
    $key = md5(__METHOD__.$id_article);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $language = $this->getAdapter()->quote($this->_locale->getLanguage());
      $where = array('id_article = ?' => $id_article);
      $order = new Zend_Db_Expr("(language = {$language}) DESC");

      $result = $this->fetchRow($where, $order);
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí článek dle $rewrite
   *
   * @param string $rewrite
   * @return System_Model_Article
   */
  public function getArticleByRewrite($rewrite) {
    $key = md5(__METHOD__.$rewrite);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $language = $this->getAdapter()->quote($this->_locale->getLanguage());
      $order = new Zend_Db_Expr("(language = {$language}) DESC");
      $result = $this->fetchRow(array('rewrite LIKE ?' => $rewrite), $order);
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí první článek
   *
   * @param string $language
   * @return System_Model_Article
   */
  public function getFirstArticle($language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }

    $key = md5(__METHOD__.$language);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $result = $this->fetchRow(array('language = ?' => $language), 'article_order ASC');
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí články dle pole $id_articles
   *
   * @param array $id_articles  Pole ID článků
   * @return Zend_Db_Table_Rowset
   */
  public function getArticles(array $id_articles, $language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }
    $where = array(
      'id_article IN (?)' => $id_articles,
      'language = ?'      => $language
    );
    return $this->fetchAll($where, 'article_order ASC');
  }

  /**
   * Vrátí seznam článků dle filtru
   *
   * @param array $filter  Kritéria pro omezení
   * @param boolean $count  Vrátit počet záznamů
   * @return Zend_Db_Table_Rowset|integer
   */
  public function getArticlesByFilter(array $filter = null, $limit = null, $offset = null, $count = false) {
    $select = $this->_select($count);
    $select->limit($limit, $offset);

    if (isset($filter['id_article']) && is_numeric($filter['id_article'])) {
      $select->where('m.id_article = ?', $filter['id_article']);
    }
    if (isset($filter['language']) && is_string($filter['language'])) {
      $select->where('a.language = ?', $filter['language']);
    }
    if (isset($filter['is_visible']) && is_bool($filter['is_visible'])) {
      $select->where('a.is_visible = ?', (int) $filter['is_visible']);
    }
    if (isset($filter['is_menuitem']) && is_bool($filter['is_menuitem'])) {
      $select->where('a.is_menuitem = ?', (int) $filter['is_menuitem']);
    }

    return $count ? (int) $this->getAdapter()->fetchOne($select) : $this->fetchAll($select);
  }

  /**
   * Vrátí počet záznamů v tabulce seskupené dle id_article
   *
   * @return integer
   */
  public function getCount() {
    $sql = "SELECT COUNT(*) FROM {$this->info('name')} GROUP BY id_article";
    return (int) $this->getAdapter()->fetchOne($sql);
  }

  /**
   * Nastaví pořadí stránek
   *
   * @param array $newOrder
   * @return void
   */
  public function reorder(array $newOrder) {
    if (is_array($newOrder) && !empty($newOrder)) {
      $tableName = $this->info(self::NAME);
      foreach ($newOrder as $order => $id) {
        $priority = $order + 1; // Indexováno od nuly
        $sql = "UPDATE {$tableName} SET article_order = "
             . "{$priority} WHERE id_article = {$id}";
        $this->getAdapter()->query($sql);
      }
    }
  }

  /**
   * Změní pořadí stránek
   *
   * @param integer $id_article
   * @param string $direction
   * @return void
   */
  public function move($id_article, $direction = 'up') {
    $tableName = $this->info(self::NAME);
    $operator = ('up' == $direction) ? '<' : '>';
    $dir = ('up' == $direction) ? 'DESC' : 'ASC';

    $fromArticle = $this->getMutationById($id_article);

    $where = array("article_order {$operator} ?" => $fromArticle->article_order);
    $order = "article_order {$dir}";

    $toArticle = $this->fetchRow($where, $order);

    $sql1 = "UPDATE {$tableName} "
          . "SET article_order = {$fromArticle->article_order} "
          . "WHERE id_article = {$toArticle->id_article}";

    $sql2 = "UPDATE {$tableName} "
          . "SET article_order = {$toArticle->article_order} "
          . "WHERE id_article = {$fromArticle->id_article}";

    $this->getAdapter()->query($sql1);
    $this->getAdapter()->query($sql2);
  }

  /**
   * Vrátí článek na základě identifikátoru
   *
   * @param mixed $identifier
   * @param string $language
   * @return System_Model_Article
   */
  public static function getArticle($identifier, $language = null) {
    $self = new self();
    return is_numeric($identifier)
           ? $self->getArticleById($identifier, $language)
           : $self->getArticleByRewrite($identifier);
  }

  /**
   * Vrátí subselect mutací článků
   *
   * @return array
   */
  private function _mutations() {
    $mutations = $this->select(self::SELECT_WITH_FROM_PART);
    $mutations->reset(self::COLUMNS)->columns(array(
      'id_article',
      'article_order',
      'GROUP_CONCAT(language) AS mutations'
    ));
    $mutations->group('id_article');
    return array('m' => new Zend_Db_Expr('('.$mutations.')'));
  }

  /**
   * Vrátí subselect alternativních názvů článků
   *
   * @param string $language
   * @return array
   */
  private function _alternatives($language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }
    $quoted = $this->getAdapter()->quote($language);

    $alternative = $this->select(self::SELECT_WITH_FROM_PART);
    $alternative->reset(self::COLUMNS)->columns(array(
      'id_article',
      'caption AS alt'
    ));
    $alternative->group('id_article');
    $alternative->order(new Zend_Db_Expr("(language = {$quoted}) DESC"));

    return array('b' => new Zend_Db_Expr('('.$alternative.')'));
  }

  /**
   * Vrátí subselect článků odpovídající jazyku $language
   *
   * @param string $language
   * @return array
   */
  private function _articles($language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }

    $articles = $this->select(self::SELECT_WITH_FROM_PART);
    $articles->where('language = ?', $language);

    return array('a' => new Zend_Db_Expr('('.$articles.')'));
  }

  /**
   * Vrátí hlavní select fitrovací funkce
   *
   * @param boolean $count  Vrátit pouze počet
   * @return Zend_Db_Table_Select
   */
  private function _select($count = false) {
    $flipped = array_flip($this->info(self::COLS));
    unset($flipped['id_article'], $flipped['article_order']);
    $columns = array_keys($flipped);

    $select = $this->select(self::SELECT_WITHOUT_FROM_PART);
    $select->setIntegrityCheck(false);
    $select->from($this->_mutations());
    $select->joinInner($this->_alternatives(), 'b.id_article = m.id_article', 'alt');
    $select->joinLeft($this->_articles(), 'a.id_article = m.id_article', $columns);
    if ($count) {
      $select->reset(self::COLUMNS)->columns('COUNT(*) AS cnt');
    } else {
      $select->order('article_order ASC');
    }

    return $select;
  }

}
