<?php

/**
 * Třída pro práci s dynamickým obsahem
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2014
 */
class System_Model_Dynamic {

  const MATCH_EXPRESSION = '\[\[\s*(.*?)\s*\]\]';

  /**
   * Reference na článek
   *
   * @var System_Model_Article
   */
  protected static $_article = null;

  /**
   * Externí drobečková navigace
   *
   * @var array
   */
  protected static $_crumbs = array();

  /**
   * Počitadlo pagerů
   *
   * @var mixed
   */
  protected static $_pagerCounter = '';

  /**
   * Nahradí syntax [[ funkce atr1=hod1 atr2=hod2 ... ]] ve článku $article dynamickým obsahem
   *
   * @param System_Model_Article $article  Článek s dynamickými vstupy
   * @param array $crumbs  Externí drobečková navigace
   * @return string
   */
  public static function content(System_Model_Article $article, array &$crumbs = array()) {
    self::$_article = $article;
    $content = preg_replace_callback(
      '#'.self::MATCH_EXPRESSION.'#isu',
      array('self', '_callback'),
      self::$_article->content
    );
    $crumbs = self::$_crumbs;
    return $content;
  }

  /**
   * Nahradí syntax [[ funkce atr1=hod1 atr2=hod2 ... ]] v $content dynamickým obsahem
   *
   * @param string $content  Obsah s dynamickými vstupy
   * @return string
   */
  public static function replace($content) {
    return preg_replace_callback(
      '#'.self::MATCH_EXPRESSION.'#isu',
      array('self', '_callback'),
      $content
    );
  }

  /**
   * Vrátí informaci o dynamickém odkazu
   *
   * @param string $content  Obsah s dynamickými vstupy
   * @return array  S klíči 'controller', 'action' a 'params'
   */
  public static function info($content) {
    $matches = array();
    if (preg_match('#'.self::MATCH_EXPRESSION.'#isu', $content, $matches)) {
      $matches[1] = preg_replace('/(%20)|(&nbsp;)/u', ' ', $matches[1]);
      $matches[1] = preg_replace('/\s*=\s*/u', '=', $matches[1]);
      $params = array_filter(preg_split('/\s/u', $matches[1]));

      $function = array_shift($params);
      $method = '_replace'.ucfirst(strtolower(trim($function)));
      if (method_exists(__CLASS__, $method)) {
        return self::$method(self::_args($params), true);
      }
    }

    return array(
      'controller' => 'index',
      'action'     => 'index',
      'params'     => array()
    );
  }

  /**
   * Vlastní metoda pro ověření shody a vrácení relevantního dynamického obsahu
   *
   * @param array $matches
   * @return mixed
   */
  protected static function _callback($matches) {
    $matches[1] = preg_replace('/(%20)|(&nbsp;)/u', ' ', $matches[1]);
    $matches[1] = preg_replace('/\s*=\s*/u', '=', $matches[1]);
    $params = array_filter(preg_split('/\s/u', $matches[1]));

    $function = array_shift($params);
    $method = '_replace'.ucfirst(strtolower(trim($function)));
    if (method_exists(__CLASS__, $method)) {
      return self::$method(self::_args($params));
    }

    return null;
  }

  /**
   * Metoda pro URL článku
   *
   * @param array $params  Parametry obsahu
   * $param boolean $asInfo
   * @return mixed
   */
  protected static function _replaceArticle(array $params, $asInfo = false) {
    if ($asInfo) {
      return array(
        'controller' => 'article',
        'action'     => 'index',
        'params'     => $params
      );
    }

    $language = self::_getLanguage(isset($params['language']) ? $params['language'] : null);
    if (isset($params['id']) && is_numeric($params['id']) &&
        is_object($article = System_Model_Articles::getArticle($params['id'], $language))) {
      return ltrim($article->getUrl(), '/');
    }

    return '';
  }

  /**
   * Metoda pro URL objektu
   *
   * @param array $params  Parametry obsahu
   * $param boolean $asInfo
   * @return string
   */
  protected static function _replaceObject(array $params, $asInfo = false) {
    if (!isset($params['size']) || !in_array($params['size'], System_Model_File::$IMAGE_SIZES)) {
      $params['size'] = System_Model_File::IMAGE_SIZE_DISPLAY;
    }

    if ($asInfo) {
      return array(
        'controller' => 'object',
        'action'     => 'index',
        'params'     => $params
      );
    }

    $language = self::_getLanguage(isset($params['language']) ? $params['language'] : null);
    if (isset($params['id']) && is_numeric($params['id']) &&
        is_object($object = System_Model_Objects::getObject($params['id'], $language)) &&
        $object->is_visible) {
      if (isset($params['action']) && ('download' == $params['action'])) {
        return ltrim($object->getUrl(), '/');
      }
      if ($object->is(System_Model_Object::SYSTEM_OBJECT_FILE) && is_object($file = $object->getFile()) && $file->isImage()) {
        return ltrim($params['size'] == System_Model_File::IMAGE_SIZE_ORIGINAL ? $object->getUrl() : $file->{$params['size']}, '/');
      }
      return ltrim($object->getUrl(), '/');
    }

    return '';
  }

  /**
   * Vrátí HTML kód galerie
   *
   * @param array $params
   *   +id       => ID galerie
   *   *language => jazyk
   *   *limit    => počet objektů
   *   *pager    => zobrazit paginator
   *   *link     => zobrazit odkaz na galerii
   * @return string
   */
  protected static function _replaceGallery(array $params) {
    $language = self::_getLanguage(isset($params['language']) ? $params['language'] : null);
    if (!isset($params['id']) || !is_numeric($params['id']) ||
        !is_object($object = System_Model_Objects::getObject($params['id'], $language)) ||
        !$object->isGallery() || !$object->is_visible) {
      return '';
    }

    $view = Zend_Registry::get('Zend_View');
    if (isset($params['limit']) && is_numeric($params['limit'])) {
      $request = Zend_Controller_Front::getInstance()->getRequest();

      $view->url = $request->getRequestUri();
      $view->page = $view->translate('System_Common_Part').self::$_pagerCounter++;

      $page = (int) $request->getParam($view->page, 1);
      $page = (1 > $page) ? 1 : $page;

      $total = $object->getMedia(true, null, null, true);
      $limit = (int) $params['limit'];

      $view->paginator = Zend_Paginator::factory($total);
      $view->paginator->setItemCountPerPage($limit);
      $view->paginator->setCurrentPageNumber($page);

      $offset = $limit * ($view->paginator->getCurrentPageNumber() - 1);

      $view->objects = $object->getMedia(true, $limit, $offset);
    } else {
      $view->objects = $object->getMedia(true);
    }

    $view->object = $object;
    $view->pager = isset($params['pager']) && ('1' == $params['pager']);
    $view->link = isset($params['link']) && ('1' == $params['link']);
    $view->caption = isset($params['caption']) && ('1' == $params['caption']);

    return $view->render('dynamic/gallery.phtml');
  }

  /**
   * Metoda pro kontaktní formulář
   *
   * @param array $params  Parametry obsahu
   * @return string
   */
  protected static function _replaceBrowser(array $params) {
    if (!isset($params['gallery'])) {
      return '';
    }

    $request = Zend_Controller_Front::getInstance()->getRequest();
    $args = $request->getParams();
    unset($args['module'], $args['controller'], $args['action'], $args['rewrite'], $args['page']);

    $view = Zend_Registry::get('Zend_View');
    $view->selfUrl = isset(self::$_article) ? self::$_article->getUrl() : '/';
    $view->size = isset($params['size']) ? $params['size'] : 'thumb';
    $view->index = isset($params['index']) && ('1' == $params['index']);

    $language = self::_getLanguage(isset($params['language']) ? $params['language'] : null);
    $filter = array(
      'language'   => $language,
      'is_gallery' => true
    );

    if ('all' != $params['gallery']) {
      $filter['id_object'] = array_map('trim', explode(',', $params['gallery']));
    }

    $modelObjects = new System_Model_Objects();
    $view->galleries = $modelObjects->getObjectsByFilter($filter);

    $page = 1;
    $view->gallery = null;
    foreach ($args as $key => $value) {
      if (is_object($view->gallery = $view->galleries->find(array('rewrite' => $key)))) {
        $page = (int) $value;
        break;
      }
    }
    if (!is_object($view->gallery) && !$view->index) {
      $view->gallery = $view->galleries->current();
    }
    if (is_object($view->gallery) && $view->gallery->isGallery()) {
      array_push(self::$_crumbs, array(
        'uri'    => $view->gallery->getUrl(),
        'label'  => $view->gallery->caption,
        'title'  => $view->gallery->caption,
        'active' => true
      ));

      $view->page = $view->gallery->rewrite;

      $limit = (0 > ($limit = isset($params['limit']) ? (int) $params['limit'] : 4)) ? 0 : $limit;
      $page = (1 > $page) ? 1 : $page;
      $total = $view->gallery->getMedia(true, null, null, true);

      $view->paginator = Zend_Paginator::factory($total);
      $view->paginator->setItemCountPerPage($limit);
      $view->paginator->setCurrentPageNumber($page);

      $offset = $limit * ($view->paginator->getCurrentPageNumber() - 1);
      $view->objects = $view->gallery->getMedia(true, $limit, $offset);
    }
    $view->captions = isset($params['captions']) && ('1' == $params['captions']);
    $view->links = isset($params['links']) && ('1' == $params['links']);

    return $view->render('dynamic/browser.phtml');
  }

  /**
   * Metoda pro kontaktní formulář
   *
   * @param array $params  Parametry obsahu
   * @return string
   */
  protected static function _replaceContactus(array $params) {
    if (!isset($params['recipients'])) {
      return '';
    }

    $validator = new Zend_Validate_EmailAddress();

    $recipients = explode(',', $params['recipients']);
    foreach ($recipients as $recipient) {
      if (!$validator->isValid($recipient) ) {
        return '';
      }
    }

    $view = Zend_Registry::get('Zend_View');
    $request = Zend_Controller_Front::getInstance()->getRequest();

    $flashMessenger = Zend_Controller_Action_HelperBroker::getStaticHelper('FlashMessenger');
    $flashMessenger->setNamespace('contactus');

    $view->form = new System_Form_ContactUs();
    $view->form->setAction($request->getRequestUri());

    if ($request->isPost() && ('contactus' === $request->getParam('postid')) && $view->form->isValid($request->getParams())) {
      System_Model_Email::notify($recipients, $view->form->getValues());
      $flashMessenger->addMessage($view->translate('System_Form_ContactUs_Sent'));

      $redirector = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
      $redirector->setGotoUrl($request->getRequestUri());
    }

    $view->messages = $flashMessenger->getMessages();
    return $view->render('dynamic/contact-us.phtml');
  }

  /**
   * Metoda pro testovací účely
   *
   * @param array $params  Parametry obsahu
   * @return string
   */
  protected static function _test(array $params) {
    $string = array();
    foreach ($params as $key => $value) {
      $string[] = htmlspecialchars($key) . ' => ' . htmlspecialchars($value);
    }
    return 'OK [' . implode(' | ', $string) . ']';
  }

  /**
   * Rozdělí páry na atributy
   *
   * @param array $params
   * @return array
   */
  private static function _args($params) {
    $args = array();
    foreach ($params as $value) {
      $attr = explode('=', $value);
      $args[reset($attr)] = trim(end($attr));
    }
    return $args;
  }

  /**
   * Vrátí dostupný jazyk
   *
   * @param string $lang
   * @return string
   */
  private static function _getLanguage($lang = null) {
    $current = Zend_Registry::get('Zend_Locale')->getLanguage();
    $languages = System_Model_Settings::getValue('languages', array('cs' => 'čeština'));
    return isset($lang) && array_key_exists($lang, $languages) ? $lang : $current;
  }

}
