<?php

/**
 * Třída pro práci s řádkem databázové tabulky webbia_files
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_File extends Zend_Db_Table_Row_Abstract {

  const IMAGE_SIZE_MINI     = 'mini';
  const IMAGE_SIZE_THUMB    = 'thumb';
  const IMAGE_SIZE_DISPLAY  = 'display';
  const IMAGE_SIZE_ORIGINAL = 'original';

  const COMMON_IMAGE_STORAGE = 'media';

  /**
   * Obrazové typy a vlstnosti
   *
   * @var array
   */
  public static $IMAGE_SIZES = array(
    self::IMAGE_SIZE_MINI,
    self::IMAGE_SIZE_THUMB,
    self::IMAGE_SIZE_DISPLAY,
    self::IMAGE_SIZE_ORIGINAL
  );

  /**
   * Název třídy objektu Zend_Db_Table_Abstract
   *
   * @var string
   */
  protected $_tableClass = 'System_Model_Files';

  /**
   * Adresář pro umístění nahrávaného souboru
   *
   * @var string
   */
  protected $_uploadDir = '';

  /**
   * Cesta k aktuálně nahrávanému souboru
   *
   * @var string
   */
  protected $_fullName = '';

  /**
   * Cesty k souborům, které jsou právě mazány
   *
   * @var array
   */
  protected $_paths = array();

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    $this->_uploadDir = dirname(THEME_PATH).'/uploads';
    if (!file_exists($this->_uploadDir)) {
      mkdir($this->_uploadDir, 0755);
      file_put_contents($this->_uploadDir.'/.htaccess', 'deny from all');
    }
  }

  /**
   * Vrátí hodnotu sloupce, pro „sloupce” file_data a real_path vrátí kýžený obsah
   *
   * @param  string $columnName           Uživatelem zadaný název sloupce
   * @return string                       Odpovídající hodnota sloupce.
   * @throws Zend_Db_Table_Row_Exception  Pokud není sloupec $columnName v řádku
   */
  public function __get($columnName) {
    if ($columnName == 'is_visible') {
      return $this->getVisibility();
    } elseif ($columnName == 'file_data') {
      return file_get_contents($this->real_path);
    } elseif ($columnName == 'real_path') {
      $extension = System_Model_FileType::getExtension($this->file_name);
      return "{$this->_uploadDir}/files/{$this->id_file}.{$extension}";
    } elseif (in_array($columnName, self::$IMAGE_SIZES)) {
      $extension = System_Model_FileType::getExtension($this->file_name);
      return $this->isImage() ? self::COMMON_IMAGE_STORAGE."/{$columnName}/{$this->file_key}.{$extension}" : '';
    } else {
      return parent::__get($columnName);
    }
  }

  /**
   * Vrátí závislý řádek modulu files, pokud existuje
   *
   * @return Files_Model_Object
   */
  public function getObject() {
    $language = Zend_Registry::get('Zend_Locale')->getLanguage();
    $select = $this->getTable()->select()->where('language = ?', $language);
    return $this->findDependentRowset('System_Model_Objects', null, $select)->current();
  }

  /**
   * Vrátí viditelnost souboru
   *
   * @return boolean
   */
  public function getVisibility() {
    if (Zend_Auth::getInstance()->hasIdentity()) {
      return true;
    }
    if (is_object($object = $this->getObject())) {
      return $object->is_visible;
    }
    return false;
  }

  /**
   * Vrátí příponu souboru
   *
   * @return string | false
   */
  public function getExtension() {
    $extensions = array_keys(System_Model_FileType::$types, $this->file_type);
    $count = count($extensions);

    if (0 == $count) {
      return System_Model_FileType::$defaultType;
    }
    if (1 == $count) {
      return reset($extensions);
    }

    $extFromFile = System_Model_FileType::getExtension($this->file_name);
    if (in_array($extension = strtolower($extFromFile), $extensions)) {
      return $extension;
    }

    return System_Model_FileType::$defaultType;
  }

  /**
   * Vrátí název souboru bez přípony
   *
   * @return string
   */
  public function getNameOnly() {
    $ext = $this->getExtension();
    return basename($this->file_name, $ext ? ".{$ext}" : '');
  }

  /**
   * Otestuje, zdali je soubor textovým souborem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public function isText() {
    return in_array($this->file_type, System_Model_FileType::$texts);
  }

  /**
   * Otestuje, zdali je soubor tabulkovým souborem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public function isTable() {
    return in_array($this->file_type, System_Model_FileType::$tables);
  }

  /**
   * Otestuje, zdali je soubor prezentačním souborem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public function isSlide() {
    return in_array($this->file_type, System_Model_FileType::$slides);
  }

  /**
   * Zkontroluje, zdali je soubor obrázkem
   *
   * @return boolean
   */
  public function isImage() {
    return in_array($this->file_type, System_Model_FileType::$images);
  }

  /**
   * Zkontroluje, zdali je soubor audiem
   *
   * @return boolean
   */
  public function isAudio() {
    return in_array($this->file_type, System_Model_FileType::$audio);
  }

  /**
   * Zkontroluje, zdali je soubor audiem (nepřehratelným v prohlížeči)
   *
   * @return boolean
   */
  public function isUnplayableAudio() {
    return in_array($this->file_type, System_Model_FileType::$unplayableAudio);
  }

  /**
   * Zkontroluje, zdali je soubor videem
   *
   * @return boolean
   */
  public function isVideo() {
    return in_array($this->file_type, System_Model_FileType::$video);
  }

  /**
   * Zkontroluje, zdali je soubor videeem (nepřehratelným v prohlížeči)
   *
   * @return boolean
   */
  public function isUnplayableVideo() {
    return in_array($this->file_type, System_Model_FileType::$unplayableVideo);
  }

  /**
   * Zkontroluje, zdali je soubor archivem
   *
   * @return boolean
   */
  public function isArchive() {
    return in_array($this->file_type, System_Model_FileType::$archives);
  }

  /**
   * Vrátí druh souboru (text / table / slide / image / audio / video / archive / unknown)
   *
   * @param boolean $all  Vracet všechy druhy (true, výchozí) nebo pouze HTML5 (false)
   * @return string
   */
  public function getKind($all = true) {
    if ($this->isText()) {
      return System_Model_FileType::FILE_KIND_TEXT;
    }
    if ($this->isTable()) {
      return System_Model_FileType::FILE_KIND_TABLE;
    }
    if ($this->isSlide()) {
      return System_Model_FileType::FILE_KIND_SLIDE;
    }
    if ($this->isImage()) {
      return System_Model_FileType::FILE_KIND_IMAGE;
    }
    if ($this->isAudio()) {
      return System_Model_FileType::FILE_KIND_AUDIO;
    }
    if ($all && $this->isUnplayableAudio()) {
      return System_Model_FileType::FILE_KIND_AUDIO;
    }
    if ($this->isVideo()) {
      return System_Model_FileType::FILE_KIND_VIDEO;
    }
    if ($all && $this->isUnplayableVideo()) {
      return System_Model_FileType::FILE_KIND_VIDEO;
    }
    if ($this->isArchive()) {
      return System_Model_FileType::FILE_KIND_ARCHIVE;
    }
    return System_Model_FileType::FILE_KIND_UNKNOWN;
  }

  /**
   * Přičte stažení souboru
   *
   * @return void
   */
  public function addDownload() {
    $table = $this->getTable()->info(Zend_Db_Table::NAME);
    $sql = "UPDATE {$table} SET download_count = download_count + 1, "
         . "date_download = NOW() WHERE id_file = {$this->id_file}";
    $this->getTable()->getAdapter()->query($sql);
  }

  /**
   * Vytvoří klíč souboru z ID, modulu, názvu a data vytvoření
   *
   * @return void
   */
  public function makeKey() {
    $table = $this->getTable();
    $adapter = $table->getAdapter();

    $sha1 = sha1($this->id_file.$this->file_name.$this->date_create);
    $key = $adapter->quote($sha1);

    $sql = "UPDATE {$table->info('name')} SET file_key "
         . "= {$key} WHERE id_file = {$this->id_file}";
    $adapter->query($sql);
    $this->_refresh();
  }

  /**
   * Pokud je soubor obrázkem, změní velikost na nastavené rozměry
   *
   * @return void
   */
  public function resizeImage() {
    if (!$this->isImage()) {
      return;
    }

    $mW = System_Model_Settings::getValue('mini_width', 128);
    $mH = System_Model_Settings::getValue('mini_height', 96);
    $tW = System_Model_Settings::getValue('thumb_width', 240);
    $tH = System_Model_Settings::getValue('thumb_height', 180);
    $dW = System_Model_Settings::getValue('display_width', 800);
    $dH = System_Model_Settings::getValue('display_height', 600);

    $angle = $this->getRotationAngle();
    $sizes = array(
      self::IMAGE_SIZE_MINI     => (object) array('w' => $mW, 'h' => $mH),
      self::IMAGE_SIZE_THUMB    => (object) array('w' => $tW, 'h' => $tH),
      self::IMAGE_SIZE_DISPLAY  => (object) array('w' => $dW, 'h' => $dH),
      self::IMAGE_SIZE_ORIGINAL => null
    );

    foreach ($sizes as $size => $dimensions) {
      if (!file_exists($dirName = dirname($fileName = dirname(THEME_PATH).'/'.$this->{$size}))) {
        mkdir($dirName, 0755);
      }
      if (is_object($dimensions)) {
        $img = System_Model_Image::fromFile($this->real_path);
        $img->resize($dimensions->w, $dimensions->w);
        $angle ? $img->rotate($angle, 0) : null;
        $img->save($fileName);
        unset($img);
      } else {
        copy($this->real_path, dirname(THEME_PATH).'/'.$this->original);
      }
    }
  }

  /**
   * Odešle obrázek přímo do prohlížeče
   *
   * @param string $size
   * @return void
   */
  public function sendImage($size) {
    if (!in_array($size, System_Model_File::$IMAGE_SIZES)) {
      $size = System_Model_File::IMAGE_SIZE_MINI;
    }
    if (!file_exists($path = dirname(THEME_PATH).'/'.$this->{$size})) {
      $this->resizeImage();
    }

    Custom_Tools::testModify(md5($this->file_key.$size));

    header('Content-Type: '.$this->file_type);
    Custom_Tools::output($path);
    exit;
  }

  /**
   * Sets all data in the row from an array.
   *
   * @param  array $data
   * @return Zend_Db_Table_Row_Abstract Provides a fluent interface
   */
  public function setFromArray(array $data) {
    if (array_key_exists('full_name', $data) && is_string($data['full_name'])) {
      $this->_fullName = $data['full_name'];
    }
    return parent::setFromArray($data);
  }

  /**
   * Vrátí natočení obrázku ve stupních dle EXIF informace
   *
   * @return integer
   */
  public function getRotationAngle() {
    if ($this->exif_ifd0_orientation) {
      switch ($this->exif_ifd0_orientation) {
        case 1:
          return 0;
        case 3:
          return 180;
        case 6:
          return -90;
        case 8:
          return 90;
      }
    }
    return 0;
  }

  /**
   * Umožní aplikovat změnu řádku před vložením.
   *
   * @return void
   */
  protected function _insert() {
    $auth = Zend_Auth::getInstance();

    $this->id_file = null;
    $this->id_user_creator = $auth->hasIdentity() ? $auth->getIdentity()->id_user : 0;
    $this->date_create = date('Y-m-d H:i:s');

    parent::_insert();
  }

  /**
   * Umožní aplikovat změnu řádku po vložení.
   *
   * @return void
   */
  protected function _postInsert() {
    parent::_postInsert();

    $this->makeKey();
    $this->_refresh();

    if ($this->_fullName) {
      if (!file_exists($dirname = dirname($this->real_path))) {
        mkdir($dirname, 0755);
      }
      rename($this->_fullName, $this->real_path);

      $this->resizeImage();
    }
  }

  /**
   * Umožní aplikovat změnu řádku před uložením.
   *
   * @return void
   */
  protected function _update() {
    parent::_update();
    $auth = Zend_Auth::getInstance();

    $this->id_user_changer = $auth->hasIdentity() ? $auth->getIdentity()->id_user : 0;
    $this->date_change = date('Y-m-d H:i:s');
  }

  /**
   * Umožní aplikovat změnu řádku před smazáním.
   *
   * @return void
   */
  public function _delete() {
    parent::_delete();

    $this->_paths[] = $this->real_path;
    foreach (self::$IMAGE_SIZES as $size) {
      if (is_string($path = $this->{$size})) {
        $this->_paths[] = dirname(THEME_PATH).$path;
      }
    }
  }

  /**
   * Umožní aplikovat změnu řádku po smazání.
   *
   * @return void
   */
  public function _postDelete() {
    foreach ($this->_paths as $path) {
      unlink($path);
    }

    parent::_postDelete();
  }

  /**
   * Vrátí adresář pro nahrání souborů
   *
   * @return string
   */
  public static function getUploadDir() {
    $file = new self();
    return $file->_uploadDir;
  }

  /**
   * Uloží soubor do databáze
   *
   * @param string $filename    Cesta k souboru
   * @return System_Model_File
   */
  public static function saveFile($filename) {
    if (!file_exists($filename)) {
      return null;
    }

    $data = array(
      'full_name' => $filename,
      'file_name' => basename($filename),
      'file_size' => filesize($filename),
      'file_type' => System_Model_FileType::getMimeType($filename)
    );
    $exif = self::_getExifInfo($filename);

    $self = new self();
    $file = $self->getTable()->createRow(array_merge($data, $exif));
    $file->save();

    return $file;
  }

  /**
   * Vrátí základní EXIF informace
   *
   * @param string $filename
   * @return array
   */
  private static function _getExifInfo($filename) {
    $result = array(
      'exif_ifd0_model'       => null,
      'exif_ifd0_orientation' => null,
      'exif_ifd0_datetime'    => null
    );

    $exif = exif_read_data($filename, null, true);
    if (false === $exif || !array_key_exists('IFD0', $exif)) {
      return $result;
    }

    $ifd0 = $exif['IFD0'];
    if (array_key_exists('Make', $ifd0) && array_key_exists('Model', $ifd0)) {
      $result['exif_ifd0_model'] = "{$ifd0['Make']} {$ifd0['Model']}";
    } elseif (array_key_exists('Make', $ifd0)) {
      $result['exif_ifd0_model'] = $ifd0['Make'];
    } elseif (array_key_exists('Model', $ifd0)) {
      $result['exif_ifd0_model'] = $ifd0['Model'];
    }
    if (array_key_exists('Orientation', $ifd0)) {
      $result['exif_ifd0_orientation'] = $ifd0['Orientation'];
    }
    if (array_key_exists('DateTime', $ifd0)) {
      list($date, $time) = preg_split('/\s+/', $ifd0['DateTime']);
      $result['exif_ifd0_datetime'] = str_replace(':', '-', $date).' '.$time;
    }

    return $result;
  }

}
