<?php

/**
 * Třída pro práci s typy souborů
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_FileType {

  const FILE_EXTENSION = '/.*\.(.*)$/';

  const FILE_KIND_TEXT    = 'text';
  const FILE_KIND_TABLE   = 'table';
  const FILE_KIND_SLIDE   = 'slide';
  const FILE_KIND_IMAGE   = 'image';
  const FILE_KIND_AUDIO   = 'audio';
  const FILE_KIND_VIDEO   = 'video';
  const FILE_KIND_ARCHIVE = 'archive';
  const FILE_KIND_UNKNOWN = 'unknown';

  /**
   * Přípony a MIME vyjádření
   *
   * @var array
   */
  public static $types = array(
    // Textové dokumenty
    'txt'  => 'text/plain',
    'rtf'  => 'application/rtf',
    'doc'  => 'application/msword',
    'dot'  => 'application/msword',
    'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'dotx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
    'odt'  => 'application/vnd.oasis.opendocument.text',
    'pdf'  => 'application/pdf',

    // Tabulkové dokumenty
    'csv'  => 'text/csv',
    'xls'  => 'application/vnd.ms-excel',
    'xlt'  => 'application/vnd.ms-excel',
    'xlsx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'xltx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
    'ods'  => 'application/vnd.oasis.opendocument.spreadsheet',

    // Prezentační dokumenty
    'ppt'  => 'application/vnd.ms-powerpoint',
    'pps'  => 'application/vnd.ms-powerpoint',
    'pot'  => 'application/vnd.ms-powerpoint',
    'pptx' => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
    'ppsx' => 'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
    'potx' => 'application/vnd.openxmlformats-officedocument.presentationml.template',
    'odp'  => 'application/vnd.oasis.opendocument.presentation',

    // Obrázky
    'bmp'  => 'image/bmp',
    'jpg'  => 'image/jpeg',
    'jpe'  => 'image/jpeg',
    'jpeg' => 'image/jpeg',
    'gif'  => 'image/gif',
    'png'  => 'image/png',

    // Audio
    'mp3'  => 'audio/mpeg',
    'wma'  => 'audio/x-ms-wma',

    // Video
    'flv'  => 'video/x-flv',
    'mp4'  => 'video/mp4',
    'wmv'  => 'video/x-ms-wmv',

    // Archivy
    'zip'  => 'application/zip',
    'rar'  => 'application/x-rar-compressed',
    '7z'   => 'application/x-7z-compressed'
  );

  /**
   * Pole MIME typů textových souborů
   *
   * @var array
   */
  public static $texts = array(
    'text/plain',
    'application/rtf',
    'application/msword',
    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
    'application/vnd.oasis.opendocument.text',
    'application/pdf'
  );

  /**
   * Pole MIME typů tabulkových souborů
   *
   * @var array
   */
  public static $tables = array(
    'text/csv',
    'application/vnd.ms-excel',
    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
    'application/vnd.oasis.opendocument.spreadsheet'
  );

  /**
   * Pole MIME typů prezentačních souborů
   *
   * @var array
   */
  public static $slides = array(
    'application/vnd.ms-powerpoint',
    'application/vnd.openxmlformats-officedocument.presentationml.presentation',
    'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
    'application/vnd.openxmlformats-officedocument.presentationml.template',
    'application/vnd.oasis.opendocument.presentation'
  );

  /**
   * Pole MIME typů obrázků
   *
   * @var array
   */
  public static $images = array(
    'image/bmp',
    'image/jpeg',
    'image/gif',
    'image/png'
  );

  /**
   * Pole MIME typů audia (pouze HTML5)
   *
   * @var array
   */
  public static $audio = array(
    'audio/mpeg'
  );

  /**
   * Pole MIME typů audia (mimo HTML5)
   *
   * @var array
   */
  public static $unplayableAudio = array(
    'audio/x-ms-wma'
  );

  /**
   * Pole MIME typů videa (pouze HTML5)
   *
   * @var array
   */
  public static $video = array(
    'video/mp4'
  );

  /**
   * Pole MIME typů videa (mimo HTML5)
   *
   * @var array
   */
  public static $unplayableVideo = array(
    'video/x-flv',
    'video/x-ms-wmv'
  );

  /**
   * Pole MIME typů archivů
   *
   * @var array
   */
  public static $archives = array(
    'application/zip',
    'application/x-rar-compressed',
    'application/x-7z-compressed'
  );

  /**
   * Výchozí typ
   *
   * @var string
   */
  public static $defaultType = 'bin';

  /**
   * Výchozí MIME typ
   *
   * @var string
   */
  public static $defaultMime = 'application/octet-stream';

  /**
   * Vrátí příponu souboru
   *
   * @param string $filename  Cesta k souboru
   * @return string
   */
  public static function getExtension($filename) {
    $matches = null;
    if (preg_match(self::FILE_EXTENSION, $filename, $matches)) {
      return strtolower($matches[1]);
    }

    return '';
  }

  /**
   * Vrátí MIME typ souboru
   *
   * @param string $filename  Cesta k souboru
   * @return string
   */
  public static function getMimeType($filename) {
    if (in_array($ext = self::getExtension($filename), array_keys(self::$types))) {
      return self::$types[$ext];
    }

    return self::$defaultMime;
  }

  /**
   * Otestuje, zdali je soubor textovým souborem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public static function isText($filename) {
    return in_array(self::getMimeType($filename), self::$texts);
  }

  /**
   * Otestuje, zdali je soubor tabulkovým souborem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public static function isTable($filename) {
    return in_array(self::getMimeType($filename), self::$tables);
  }

  /**
   * Otestuje, zdali je soubor prezentačním souborem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public static function isSlide($filename) {
    return in_array(self::getMimeType($filename), self::$slides);
  }

  /**
   * Otestuje, zdali je soubor obrázkem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public static function isImage($filename) {
    return in_array(self::getMimeType($filename), self::$images);
  }

  /**
   * Otestuje, zdali je soubor audiem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public static function isAudio($filename) {
    return in_array(self::getMimeType($filename), self::$audio);
  }

  /**
   * Otestuje, zdali je soubor videem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public static function isVideo($filename) {
    return in_array(self::getMimeType($filename), self::$video);
  }

  /**
   * Otestuje, zdali je soubor archivem
   *
   * @param string $filename  Cesta k souboru
   * @return boolean
   */
  public static function isArchive($filename) {
    return in_array(self::getMimeType($filename), self::$archives);
  }

  /**
   * Vrátí druh souboru (text / table / slide / image / audio / video / archive / unknown)
   *
   * @param string $filename  Cesta k souboru
   * @return string
   */
  public static function getFileKind($filename) {
    if (self::isText($filename)) {
      return self::FILE_KIND_TEXT;
    }
    if (self::isTable($filename)) {
      return self::FILE_KIND_TABLE;
    }
    if (self::isSlide($filename)) {
      return self::FILE_KIND_SLIDE;
    }
    if (self::isImage($filename)) {
      return self::FILE_KIND_IMAGE;
    }
    if (self::isAudio($filename)) {
      return self::FILE_KIND_AUDIO;
    }
    if (self::isVideo($filename)) {
      return self::FILE_KIND_VIDEO;
    }
    if (self::isArchive($filename)) {
      return self::FILE_KIND_ARCHIVE;
    }
    return self::FILE_KIND_UNKNOWN;
  }

  /**
   * Vrátí možnosti pro výběr typu souboru
   *
   * @param string $type  Typ souborů (image, media, file)
   * @return array
   */
  public static function getOptions($type) {
    $options = array();
    $view = Zend_Registry::get('Zend_View');

    if (in_array($type, array('image', 'media'))) {
      $images = self::$images;

      $allTypes = implode(',', $images);
      $allTitle = $view->translate('System_Common_All_Images');

      $options[0] = $allTitle;
      $options += array_combine($images, $images);
    }

    if ('file' == $type) {
      $types = self::$types;

      $allTypes = implode(',', $types);
      $allTitle = $view->translate('System_Common_All_Files');

      $options[0] = $allTitle;
      $options += array_combine($types, $types);
    }

    return $options;
  }

}
