<?php

/**
 * Třída pro přihlášení nebo ohlášení uživatele
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class System_Model_Log {

  const ALLOWED_ATTEMPTS = 3;

  /**
   * Session pro počty pokusů přihlášení
   *
   * @var Zend_Session_Namespace
   */
  private static $_attempts = null;

  /**
   * Vrátí formulář pro přihlášení nebo přihlásí uživatele
   *
   * @param string $actionUrl  URL adresa pro kontrolu přihlašovacího formuláře
   * @param string $redirectUrl  URL adresa pro přesměrování přihlášeného uživatele
   * @param array $messages  Zprávy o přihlášení (pouze na výstup)
   * @param string $type  Typ přihlášení (database|opeid)
   * @return Zend_Form
   * @throws Zend_Exception
   */
  public static function in($actionUrl, $redirectUrl, &$messages = null, $type = 'database') {
    self::_initAttempts();

    $auth = Zend_Auth::getInstance();
    $front = Zend_Controller_Front::getInstance();
    $view = Zend_Registry::get('Zend_View');

    $request = $front->getRequest();
    $response = $front->getResponse();
    $params = $request->getParams();

    $form = null;
    if ('database' == $type) {
      $form = new System_Form_Log();
    }
    if ('openid' == $type) {
      $form = new System_Form_OpenID();
    }
    if ('otp' == $type) {
      $form = new System_Form_Otp();
    }
    if (null === $form) {
      throw new Zend_Exception('Nepodporovaný typ přihlášení');
    }
    $form->setAction($actionUrl);

    $remoteIP = $request->getServer('REMOTE_ADDR');
    if (self::_testForBlock($remoteIP)) {
      $messages[] = array('warning' => 'Příliš mnoho pokusů o přihlášení. Počkejte chvíli…');
      return $form;
    }

    if ('database' == $type && $request->isPost() && $form->isValid($params)) {
      $data = $form->getValues();

      $modelUsers = new System_Model_Users();
      $user = $modelUsers->userExists($data['user'], $data['pass'], array('is_active = ?' => (int) true));
      if (is_object($user)) {
        $data['permanent'] && System_Model_Logins::login($user->id_user);

        if (self::_testForOTP($user, $response)) {
          exit;
        }

        $auth->getStorage()->write($user->hidePassword());
        self::_freeAttempts();

        $message = array('info' => $view->translate('System_Admin_Logged_In'));
        $flashMessenger = Zend_Controller_Action_HelperBroker::getStaticHelper('FlashMessenger');
        $flashMessenger->addMessage($message);

        $response->setRedirect($redirectUrl);
      } else {
        if (++self::$_attempts->database > self::ALLOWED_ATTEMPTS) {
          self::_blockIP($remoteIP);
          $messages[] = array('warning' => 'Další pokus o přihlášení bude možný za 30 min.');
        } else {
          $messages[] = array('error' => $view->translate('invalidUser'));
        }
      }
    }

    if ('openid' == $type) {
      $authenticate = false;

      $session = new Zend_Session_Namespace('Login');
      $profile = new Zend_OpenId_Extension_Sreg(array('email' => true), null, 1.1);

      $authAdapter = new Zend_Auth_Adapter_OpenId();
      $authAdapter->setReturnTo($actionUrl.'?type=openid');
      $authAdapter->setExtensions($profile);

      if ($request->isPost() && $form->isValid($params)) {
        $session->permanent = $form->getValue('permanent');
        $session->setExpirationHops(1, 'permanent');
        $authAdapter->setIdentity($form->getValue('openid_identifier'));
        $authenticate = true;
      }
      if ($request->isGet() && array_key_exists('openid_mode', $params)) {
        $authenticate = true;
      }
      if ($authenticate) {
        $result = $authAdapter->authenticate();
        if ($result->isValid()) {
          $info = $profile->getProperties();

          $modelUsers = new System_Model_Users();
          if (is_object($user = $modelUsers->getUserByEmail($info['email']))) {
            $auth->getStorage()->write($user->hidePassword());
          }

          $data['permanent'] && System_Model_Logins::login($user->id_user);

          $response->setRedirect($redirectUrl);
        } else {
          $messages = array();
          foreach ($result->getMessages() as $message) {
            $messages[] = array('error' => $message);
          }
          if (++self::$_attempts->openid > self::ALLOWED_ATTEMPTS) {
            self::_blockIP($remoteIP);
            $messages[] = array('warning' => 'Další pokus o přihlášení bude možný za 30 min.');
          }
        }
      }
    }

    if ('otp' == $type && $request->isPost() && $form->isValid($params)) {
      $flashMessenger = Zend_Controller_Action_HelperBroker::getStaticHelper('FlashMessenger');

      $modelUsers = new System_Model_Users();
      $session = new Zend_Session_Namespace('otp');

      if (!is_object($user = $modelUsers->getUserById($session->user))) {
        $message = array('error' => $view->translate('invalidUser'));
        $flashMessenger->addMessage($message);
        $response->setRedirect($actionUrl)->sendResponse();
        exit;
      }

      if ($user->verifyOTP($form->getValue('otp'))) {
        $user->save(); // nutno uložit uživatelovo počítadlo
        $auth->getStorage()->write($user->hidePassword());

        $message = array('info' => $view->translate('System_Admin_Logged_In'));
        $flashMessenger->addMessage($message);

        $response->setRedirect($redirectUrl);
      } else {
        if (++self::$_attempts->otp > self::ALLOWED_ATTEMPTS) {
          self::_blockIP($remoteIP);
          $messages[] = array('warning' => 'Další pokus o přihlášení bude možný za 30 min.');
        } else {
          $session->user = $user->id_user;
          $session->setExpirationHops(1, 'user', true);
        }
      }
    }

    return $form;
  }

  /**
   * Odhlásí uživatele - vrátí true, pokud byl odhlášen; jinak false
   *
   * return boolean
   */
  public static function out() {
    $auth = Zend_Auth::getInstance();
    $auth->clearIdentity();

    System_Model_Logins::logout();
    self::_freeAttempts();

    return !$auth->hasIdentity();
  }

  /**
   * Nastaví session pro počty pokusů
   *
   * @return void
   */
  private static function _initAttempts() {
    if (null === self::$_attempts) {
      self::$_attempts = new Zend_Session_Namespace('attempts');
    }
    if (!isset(self::$_attempts->database)) {
      self::$_attempts->database = 1;
    }
    if (!isset(self::$_attempts->openid)) {
      self::$_attempts->openid = 1;
    }
    if (!isset(self::$_attempts->openid)) {
      self::$_attempts->otp = 1;
    }
  }

  /**
   * Uvolní session počtu pokusů
   *
   * @return void
   */
  private static function _freeAttempts() {
    if (null !== self::$_attempts) {
      self::$_attempts->unsetAll();
    }
    self::$_attempts = null;
  }

  /**
   * Otestuje, má-li uživatel nastaveno přihlašování přes OTP,
   * případně přesměruje uživatele na stránku s přihlášením
   *
   * @param System_Model_User $user
   * @param Zend_Controller_Response_Abstract $response
   * @return boolean
   */
  private static function _testForOTP(System_Model_User $user, Zend_Controller_Response_Abstract $response) {
    $url = Zend_Controller_Action_HelperBroker::getStaticHelper('Url');

    $hasOTP = (null !== $user->otp_type);
    if ($hasOTP) {
      $session = new Zend_Session_Namespace('otp');
      $session->user = $user->id_user;
      $session->setExpirationHops(1, 'user', true);

      if ($user->otp_seed) {
        $response->setRedirect($url->simple('login', 'admin', 'system', array('type' => 'otp')))->sendResponse();
      } else {
        $response->setRedirect($url->simple('otp', 'admin'))->sendResponse();
      }
    }
    return $hasOTP;
  }

  /**
   * Otestuje, zdali je nastavena IP pro blokování
   *
   * @param string $ip
   * @return boolean
   */
  private static function _testForBlock($ip) {
    return System_Model_Blocked::isBlocked($ip);
  }

  /**
   * Nastaví blokovanou IP adresu
   *
   * @param string $ip
   * @return void
   */
  private static function _blockIP($ip) {
    self::_freeAttempts();
    System_Model_Blocked::blockIP($ip, 30 * 60); // 30 min.
  }

}
