<?php

/**
 * Třída pro práci s řádkem databázové tabulky webbia_objects
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Object extends Custom_Db_Table_Row_Rewrite
                          implements Custom_Search_Interface {

  const SYSTEM_OBJECT_FOLDER = 'folder';
  const SYSTEM_OBJECT_FILE   = 'file';

  const ATTR_IS_VISIBLE = 'is_visible';
  const ATTR_IS_GALLERY = 'is_gallery';

  /**
   * Typy objektů
   *
   * @var array
   */
  public static $OBJECT_TYPES = array(
    self::SYSTEM_OBJECT_FOLDER => 'System_Object_Folder',
    self::SYSTEM_OBJECT_FILE   => 'System_Object_File'
  );

  /**
   * Název třídy objektu Zend_Db_Table_Abstract
   *
   * @var string
   */
  protected $_tableClass = 'System_Model_Objects';

  /**
   * Pořadí nového objektu
   *
   * @var integer
   */
  private $_newOrder = null;

  /**
   * Vrátí hodnotu sloupce, pro „sloupce” size a type vrátí kýžený obsah
   *
   * @param  string $columnName  Uživatelem zadaný název sloupce
   * @return string              Odpovídající hodnota sloupce.
   * @throws Zend_Db_Table_Row_Exception  Pokud není sloupec $columnName v řádku
   */
  public function __get($columnName) {
    if (in_array($columnName, array('size', 'type', 'kind', 'ext')) && !array_key_exists($columnName, $this->_data)) {
      if ($columnName == 'size') {
        return $this->getSize();
      }
      if ($columnName == 'type') {
        return is_object($file = $this->getFile()) ? $file->file_type : self::SYSTEM_OBJECT_FOLDER;
      }
      if ($columnName == 'kind') {
        return is_object($file = $this->getFile()) ? $file->getKind() : self::SYSTEM_OBJECT_FOLDER;
      }
      if ($columnName == 'ext') {
        return is_object($file = $this->getFile()) ? $file->getExtension() : self::SYSTEM_OBJECT_FOLDER;
      }
    }
    return parent::__get($columnName);
  }

  /**
   * Nastaví všechna data objektu z pole
   *
   * @param  array $data
   * @return Zend_Db_Table_Row_Abstract  Poskytuje fluent zápis
   */
  public function setFromArray(array $data) {
    $tmp = array('_controller' => 'object', '_action' => 'index');
    return parent::setFromArray($data + $tmp);
  }

  /**
   * Vrátí drobečnovou navigaci objektu
   *
   * @return array  Asociativní pole hodnot drobečkové navigace
   */
  public function getBreadcrumbs() {
    $front = Zend_Controller_Front::getInstance();
    $ctrl = $front->getRequest()->getControllerName();

    $crumb  = array('label' => $this->caption, 'title' => $this->caption);
    $crumb += ($ctrl == 'index')
            ? array('uri' => $this->getUrl(), 'active' => true)
            : array(
                'controller' => $front->getRequest()->getControllerName(),
                'action'     => $front->getRequest()->getActionName(),
                'params'     => array('id' => $this->id_object),
                'active'     => true
              );

    return array($crumb);
  }

  /**
   * Zkontroluje typ objektu
   *
   * @param string $type  Jedna z možností (file, folder)
   * @return boolean      TRUE, odpovídá-li typ objektu paramentru; jinak FALSE
   */
  public function is($type) {
    $result = false;
    if (array_key_exists($type, self::$OBJECT_TYPES)) {
      $result = ($this->object_type == $type);
    }
    return $result;
  }

  /**
   * Vrátí objekt souboru
   *
   * @return System_Model_File|null
   */
  public function getFile() {
    if (self::SYSTEM_OBJECT_FOLDER == $this->object_type) {
      return null;
    }

    if (!$this->_connected) {
      $this->_table = new $this->_tableClass();
      $this->_connected = is_object($this->_table);
    }

    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = md5('System_Model_Files::getFileById'.$this->id_file);

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $result = $this->findParentRow('System_Model_Files');
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }
    if (null === $result && isset($this->alt_file)) {
      $result = System_Model_Files::getFile($this->alt_file);
    }

    return $result;
  }

  /**
   * Vrátí velikost objektu
   *
   * @return integer
   */
  public function getSize() {
    if (self::SYSTEM_OBJECT_FOLDER == $this->object_type) {
      return $this->_getFolderSize();
    }
    if (self::SYSTEM_OBJECT_FILE == $this->object_type) {
      return is_object($file = $this->getFile()) ? $file->file_size : 0;
    }
  }

  /**
   * Vrátí typ objektu - složka, přípona souboru
   *
   * @return string|null
   */
  public function getType() {
    if ($this->is(self::SYSTEM_OBJECT_FOLDER)) {
      return self::SYSTEM_OBJECT_FOLDER;
    }
    if ($this->is(self::SYSTEM_OBJECT_FILE)) {
      return is_object($file = $this->getFile()) ? $file->getExtension() : null;
    }
    return null;
  }

  /**
   * Vrátí rodičovský objekt
   *
   * @param  boolean $visible_only  Zobrazovat pouze viditelné (výchozí NULL)
   * @return System_Model_Object|null
   */
  public function getParent($visible_only = null) {
    if (null === $this->id_parent) {
      return null;
    }

    $object = is_object($table = $this->getTable())
            ? $table->getObjectById($this->id_parent)
            : System_Model_Objects::getObject($this->id_parent);
    if (is_bool($visible_only) && $visible_only && is_object($object) && !$object->is_visible) {
      $object = null;
    }

    return $object;
  }

  /**
   * Vrátí objekty pro galerii
   *
   * @param boolean $is_visible  Pouze viditelné (výchozí TRUE)
   * @param integer $limit  Omezení počtu souborů
   * @param integer $offset  Posunutí začátku souborů
   * @param boolean $count  Vrátit pouze počet (výchozí FALSE)
   * @return System_Model_ObjectSet
   */
  public function getMedia($is_visible = true, $limit = null, $offset = null, $count = false) {
    return $this->getTable()->getMedia($this->id_object, $is_visible, $this->language, $limit, $offset, $count);
  }

  /**
   * Vrátí potomky aktuálního objektu a aktuální mutace
   *
   * @param array $where     Kritéria objektů
   * @param boolean $limit   Počet objektů (výchozí NULL)
   * @param boolean $offset  Offset objektu (výchozí NULL)
   * @param boolean $count   Vrátit počet článků (výchozí FALSE)
   * @return System_Model_ObjectSet|integer  Skupina řádků s objekty
   */
  public function getDescendants(array $where = null, $limit = null, $offset = null, $count = false) {
    if (null === $where) {
      $where = array();
    }
    $where['id_parent = ?'] = $this->id_object;
    $where['language LIKE ?'] = $this->language;

    $table = $this->getTable();
    array_walk($where, function ($value, $cond, $select) {
      is_numeric($cond) ? $select->where($value) : $select->where($cond, $value);
    }, $select = $table->select(true));

    if ($count) {
      $select->reset('columns')->columns('COUNT(*)');
      return (int) $table->getAdapter()->fetchOne($select);
    }

    $select->order('object_order ASC')->limit($limit, $offset);
    return $table->fetchAll($select);
  }

  /**
   * Vrátí TRUE, pokud je $id_tested_node potomkem tohoto objektu
   *
   * @param integer $id_tested_node  Testované ID
   * @return boolean  TRUE, pokud je potomkem; jinak FALSE
   */
  public function isDescendant($id_tested_node) {
    if (is_object($object = $this->getTable()->getObjectById($id_tested_node))) {
      return ($object->id_parent == $this->id_object);
    }
    return false;
  }

  /**
   * Vrátí TRUE, pokud je objekt složkou a zároveň galerií
   *
   * @return boolean  TRUE, pokud je galerií; jinak FALSE
   */
  public function isGallery() {
    return $this->is(self::SYSTEM_OBJECT_FOLDER) && $this->is_gallery;
  }

  /**
   * Vrátí první objekt ve složce
   *
   * @param boolean $image_only  Pouze obrázky
   * @return System_Model_Object|null
   */
  public function getFirst($image_only = true) {
    return $this->_getBoundary('first', $image_only);
  }

  /**
   * Vrátí poslední objekt ve složce
   *
   * @param boolean $image_only  Pouze obrázky
   * @return System_Model_Object|null
   */
  public function getLast($image_only = true) {
    return $this->_getBoundary('last', $image_only);
  }

  /**
   * Vrátí TRUE, pokud je objekt první ve složce; jinak FALSE
   *
   * @param boolean $as_image
   * @return boolean
   */
  public function isFirst($as_image = true) {
    if (!is_object($parent = $this->getParent())) {
      return false;
    }
    if (is_object($first = $parent->getFirst($as_image))) {
      return ($this->id_object = $first->id_object);
    }
    return false;
  }

  /**
   * Vrátí TRUE, pokud je objekt poslední ve složce; jinak FALSE
   *
   * @param boolean $as_image
   * @return boolean
   */
  public function isLast($as_image = true) {
    if (!is_object($parent = $this->getParent())) {
      return false;
    }
    if (is_object($last = $parent->getLast($as_image))) {
      return ($this->id_object = $last->id_object);
    }
    return false;
  }

  /**
   * Vrátí předchozí objekt na stejné úrovni
   *
   * @return Files_Model_Object|null
   */
  public function getPrevious() {
    return is_object($obj = $this->_getNeighbour('prev')) && $obj->is_visible
           ? $obj
           : null;
  }

  /**
   * Vrátí následující objekt na stejné úrovni
   *
   * @return Files_Model_Object|null
   */
  public function getNext() {
    return is_object($obj = $this->_getNeighbour('next')) && $obj->is_visible
           ? $obj
           : null;
  }

  /**
   * Vrátí URL objektu
   *
   * @param string $size
   * @return string  URL objektu
   */
  public function getUrl($size = System_Model_File::IMAGE_SIZE_DISPLAY) {
    $baseUrl = Zend_Controller_Front::getInstance()->getBaseUrl();

    if ($size && $this->is(self::SYSTEM_OBJECT_FILE) &&
        is_object($file = $this->getFile()) && $file->isImage()) {
      $url = $this->is_visible ? $file->$size : '';
      if ($baseUrl) {
        $url = $baseUrl . '/' . $url;
      }
      if (!file_exists(dirname(THEME_PATH).'/'.$url)) {
        $file->resizeImage();
      }
      return $url;
    }

    $url = $this->is_visible ? $this->rewrite : '';

    $object = clone $this;
    while (is_object($object = $object->getParent())) {
      if (!$object->is_visible) {
        return '';
      }
      $url = $object->rewrite.'/'.$url;
    }
    if ($baseUrl) {
      $url = $baseUrl . '/' . $url;
    }

    return $url;
  }

  /**
   * Vrátí informaci o objektu pro bublinovou nápovědu
   *
   * @return string
   */
  public function getToolTip() {
    $view = Zend_Registry::get('Zend_View');

    $nl = "     \n";
    $sep = ': ';
    $tooltip = '';
    $dateFormat = $view->translate('System_Common_Full_Datetime');

    $created = $view->translate('System_Common_Created_By')
             . $sep
             . (is_object($creator = $view->user($this->id_user_creator)) ? $creator->getFullName().', ' : '')
             . $view->date($dateFormat, $this->date_create);
    $tooltip .= $created.$nl;

    if ($this->date_change) {
      $changed = $view->translate('System_Common_Changed_By')
               . $sep
               . (is_object($changer = $view->user($this->id_user_changer)) ? $changer->getFullName().', ' : '')
               . $view->date($dateFormat, $this->date_change);
      $tooltip .= $changed.$nl;
    }
    if ($this->is(self::SYSTEM_OBJECT_FOLDER)) {
      if ($this->size) {
        $tooltip .= $view->translate('System_Object_Size').$sep.$view->fileSize($this->size).$nl;
      } else {
        $tooltip .= $view->translate('System_Object_Empty_Folder').$nl;
      }
      $question = Zend_Registry::get('Zend_Locale')->getQuestion();
      $tooltip .= $view->translate('System_Object_Is_Gallery').$sep.$question[$this->is_gallery ? 'yes': 'no'];
    }
    if ($this->is(self::SYSTEM_OBJECT_FILE)) {
      $tooltip .= $view->translate('System_Object_File').$sep.$view->escape($this->caption).$nl;
      $tooltip .= $view->translate('System_Object_Size').$sep.$view->fileSize($this->size).$nl;
      $tooltip .= $view->translate('System_Object_Type').$sep.$view->escape($this->type).$nl;
    }

    return $tooltip;
  }

  /**
   * Vrátí kód URL objektu
   *
   * @return string  URL objektu
   */
  public function getCode() {
    return "[[object id={$this->id_object}]]";
  }

  /**
   * Vrátí data pro zaindexování do Zend_Search_Lucene
   *
   * @return array  Asociativní pole s akcí a daty
   */
  public function getDocuments() {
    $this->_refresh();

    $id_document = get_class($this->getTable()).':'.$this->id_object.':'.$this->language;

    $document = new Zend_Search_Lucene_Document();
    $document->addField(Zend_Search_Lucene_Field::keyword('id_document', $id_document));
    $document->addField(Zend_Search_Lucene_Field::text('caption', $this->caption, 'UTF-8'));

    if (null !== $this->description) {
      $htmlDescription = Zend_Search_Lucene_Document_Html::loadHTML($this->description, true, 'UTF-8');
      $description = $htmlDescription->getField('body');
      $description->name = 'content';
      $document->addField($description);
    }

    $key = ($this->is_visible)
         ? Custom_Search_Interface::INDEX_ACTION_UPDATE
         : Custom_Search_Interface::INDEX_ACTION_REMOVE;

    $documents = array(
      array($key => $document)
    );

    return $documents;
  }

  /**
   * Vrátí čitelnost objektu
   *
   * @return boolean  TRUE v případě čitelnosti, jinak FALSE
   */
  public function isReadable() {
    $readable = true;
    if (!Zend_Auth::getInstance()->hasIdentity() && !$this->is_visible) {
      $readable = false;
    }
    return $readable;
  }

  /**
   * Vymaže existující řádek z databáze
   *
   * @return integer  Počet vymazaných řádků
   */
  public function delete() {
    $descendants = $this->getDescendants();
    foreach ($descendants as $descendant) {
      $descendant->delete();
    }
    if (is_object($file = $this->getFile()) && (1 == $this->_getFileCount($this->id_file))) {
      $file->delete();
    }
    return parent::delete();
  }

  /**
   * Umožní aplikovat změnu řádku před vložením.
   *
   * @return void
   */
  protected function _insert() {
    $auth = Zend_Auth::getInstance();
    $this->id_user_creator = $auth->hasIdentity() ? $auth->getIdentity()->id_user : 0;
    if (!is_numeric($this->id_object)) {
      $this->id_object = null;
    }
    if (null === $this->date_create) {
      $this->date_create = date('Y-m-d H:i:s');
    }
    $this->_newOrder = $this->_getOrder();

    parent::_insert();
  }

  /**
   * Umožní aplikovat změnu řádku po vložení.
   *
   * @return void
   */
  protected function _postInsert() {
    parent::_postInsert();

    $this->_updateOrder();
    $this->_afterSave();
  }

  /**
   * Umožní aplikovat změnu řádku před uložením.
   *
   * @return void
   */
  protected function _update() {
    $auth = Zend_Auth::getInstance();
    $this->id_user_changer = $auth->hasIdentity() ? $auth->getIdentity()->id_user : 0;
    $this->date_change = date('Y-m-d H:i:s');

    parent::_update();
  }

  /**
   * Umožní aplikovat změnu řádku po uložení.
   *
   * @return void
   */
  protected function _postUpdate() {
    parent::_postUpdate();

    $this->_afterSave();
  }

  /**
   * Opraví pořadí
   *
   * @return void
   */
  protected function _updateOrder() {
    $table = $this->getTable();
    $new_order = (null === $this->_newOrder) ? $this->id_object : $this->_newOrder;

    $sql = "UPDATE {$table->info('name')} SET object_order = "
         . "{$new_order} WHERE id_object = {$this->id_object}";
    $table->getAdapter()->query($sql);
  }

  /**
   * Rutiny po vložení nebo uložení do databáze - opraví id_parent a vymaže MediaCache
   *
   * @return void
   */
  protected function _afterSave() {
    $table = $this->getTable();
    $id_parent = (null === $this->id_parent) ? 'NULL' : $this->id_parent;

    $sql = "UPDATE {$table->info('name')} SET id_parent = "
         . "{$id_parent} WHERE id_object = {$this->id_object}";
    $table->getAdapter()->query($sql);

    $table->cleanMediaCache();
  }

  /**
   * Vrátí velikost složky v bytech
   *
   * @return integer
   */
  protected function _getFolderSize() {
    $modelObjects = $this->getTable();
    $modelFiles = new System_Model_Files();

    $sql = "SELECT SUM(file_size) FROM {$modelFiles->info('name')} WHERE "
         . "id_file IN (SELECT id_file FROM {$modelObjects->info('name')} "
         . "WHERE id_file IS NOT NULL AND id_parent = {$this->id_object})";
    return (int) $this->getTable()->getAdapter()->fetchOne($sql);
  }

  /**
   * Vrátí pořadí objektu
   *
   * @return integer
   */
  private function _getOrder() {
    if (is_numeric($this->id_object) &&
        is_object($object = $this->getTable()->getMutationById($this->id_object))) {
      return $object->object_order;
    }
    return null;
  }

  /**
   * Vrátí nejbližší objekt na stejné úrovni
   *
   * @param string $direction - následující (next), předchozí (prev)
   * @return System_Model_Object|null
   */
  private function _getNeighbour($direction = 'next') {
    $operator = ('next' == $direction) ? '>' : '<';
    $order = ('next' == $direction) ? 'ASC' : 'DESC';

    $locale = Zend_Registry::get('Zend_Locale');
    $table = $this->getTable();

    $where = array(
      'object_type LIKE ?'                => self::SYSTEM_OBJECT_FILE,
      "object_order {$operator} ?"        => $this->object_order,
      '(language IS NULL OR language = ?' => $locale->getLanguage()
    );

    if (is_numeric($id_parent = $this->id_parent)) {
      $where['id_parent = ?'] = $id_parent;
    } else {
      $where[] = 'id_parent IS NULL';
    }

    if (is_object($row = $table->fetchRow($where, "object_order {$order}")) &&
        is_object($file = $row->getFile()) &&
        $file->isImage()) {
      return $row;
    }
    return is_object($row) ? $this->_getNeighbour($direction) : null;
  }

  /**
   * Vrátí mezní objekt
   *
   * @param string $bound  Okraj - 'first', 'last'
   * @param boolean $image_only  Pouze obrázky
   * @return System_Model_Object|null
   */
  private function _getBoundary($bound = 'first', $image_only = true) {
    $where = array(
      'language LIKE ?' => $this->language,
      'object_type = ?' => self::SYSTEM_OBJECT_FILE
    );
    if (is_numeric($id_parent = $this->id_object)) {
      $where['id_parent = ?'] = $id_parent;
    } else {
      $where[] = 'id_parent IS NULL';
    }
    $order = 'object_order '.(('first' == $bound) ? 'ASC' : 'DESC');

    $result = $this->getTable()->fetchRow($where, $order);
    if ($image_only && is_object($result) && is_object($file = $result->getFile()) && !$file->isImage()) {
      $result = ('first' == $bound) ? $result->getNext() : $result->getPrevious();
    }
    return $result;
  }

  /**
   * Vrátí počet vazeb na soubor
   *
   * @param integer $id_file
   * @return integer
   */
  private function _getFileCount($id_file) {
    $table = $this->getTable();
    $sql = "SELECT COUNT(id_file) FROM {$table->info('name')} WHERE id_file = {$id_file}";
    return (int) $table->getAdapter()->fetchOne($sql);
  }

}
