<?php

/**
 * Třída modelu pro vyhledávání
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class System_Model_Search {

  /**
   * Index Zend_Search_Lucene
   *
   * @var Zend_Search_Lucene
   */
  protected $_index = null;

  /**
   * Adresář indexů
   *
   * @var string
   */
  protected $_indexDirectory = '';

  /**
   * Konstruktor
   *
   * @param string $path
   * @return void
   */
  public function __construct($path = 'indexes') {
    $this->_indexDirectory = THEME_PATH."/{$path}";

    $method = file_exists("{$this->_indexDirectory}/segments.gen") ? 'open' : 'create';
    $this->_index = Zend_Search_Lucene::$method($this->_indexDirectory);

    Zend_Search_Lucene_Analysis_Analyzer::setDefault(
      new Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8_CaseInsensitive()
    );
  }

  /**
   * Vyhledá řetězec $for v indexu
   *
   * @param string $for
   * @return array
   */
  public function search($for) {
    $query = Zend_Search_Lucene_Search_QueryParser::parse($for, 'UTF-8');
    return $this->_index->find($query);
  }

  /**
   * Vrátí indexované termíny
   *
   * @param string $term
   * @return array
   */
  public function getTerms($term = null) {
    $cache = Zend_Registry::isRegistered('Zend_Cache') ? Zend_Registry::get('Zend_Cache') : null;
    $key = md5('Search_Terms');

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $result = $this->_termsToArray();
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }

    if (!empty($term)) {
      $callback = function ($item) use ($term) {
        return (0 === strpos($item, $term));
      };
      $result = array_values(array_filter($result, $callback));
    }

    return $result;
  }

  /**
   * Přegeneruje index
   *
   * @return boolean
   */
  public function rebuild() {
    $result = true;

    $classes = get_declared_classes();
    foreach ($classes as $class) {
      set_time_limit(0);
      $interfaces = class_implements($class);
      if (is_subclass_of($class, 'Zend_Db_Table_Row_Abstract') &&
          in_array('Custom_Search_Interface', $interfaces)) {
        $result = $result && $this->_recordRow(new $class());
      }
    }

    if ($result) {
      $this->_index->optimize();

      $key = md5('Search_Terms');
      $cache = Zend_Registry::isRegistered('Zend_Cache')
             ? Zend_Registry::get('Zend_Cache')
             : null;
      if (is_object($cache)) {
        $cache->save($this->_termsToArray(), $key);
      }
    }

    return $result;
  }

  /**
   * Přidá dokument do indexu
   *
   * @param Zend_Search_Lucene_Document $document
   */
  protected function _addDocument(Zend_Search_Lucene_Document $document) {
    $this->_removeDocument($document);

    $this->_index->addDocument($document);
    $this->_index->commit();
  }

  /**
   * Zruší dokument v indexu
   *
   * @param Zend_Search_Lucene_Document $document
   */
  protected function _removeDocument(Zend_Search_Lucene_Document $document) {
    $term = new Zend_Search_Lucene_Index_Term($document->getFieldValue('id_document'), 'id_document');
    $query = new Zend_Search_Lucene_Search_Query_Term($term);

    $hits = $this->_index->find($query);
    if (count($hits)) {
      foreach ($hits as $hit) {
        $this->_index->delete($hit->id);
      }
      $this->_index->commit();
    }
  }

  /**
   * Vrátí všechny indexované termíny delší než 2 znaky do pole
   *
   * @return array
   */
  protected function _termsToArray() {
    $this->_index = Zend_Search_Lucene::open($this->_indexDirectory);
    $terms = array();
    foreach ($this->_index->terms() as $term) {
      if (('id_document' != $term->field) && (2 < strlen($term->text))) {
        $terms[] = $term->text;
      }
    }
    sort($terms);
    return array_unique($terms);
  }

  /**
   * Vloží nebo zruší záznam o dokumentu v indexu
   *
   * @param string $event
   * @param Custom_Search_Interface $row
   * @return boolean
   */
  public static function record($event, $row) {
    if (!$row instanceof Custom_Search_Interface) {
      return false;
    }

    $self = new self();
    switch ($event) {
      case Custom_Db_Table_Row_Observer::OBSERVE_POST_INSERT:
      case Custom_Db_Table_Row_Observer::OBSERVE_POST_UPDATE:
        $documents = $row->getDocuments();

        foreach ($documents as $document) {
          if (isset($document[Custom_Search_Interface::INDEX_ACTION_UPDATE])) {
            $update = $document[Custom_Search_Interface::INDEX_ACTION_UPDATE];
            if ($update instanceof Zend_Search_Lucene_Document) {
              $self->_addDocument($update);
            }
          }

          if (isset($document[Custom_Search_Interface::INDEX_ACTION_REMOVE])) {
            $remove = $document[Custom_Search_Interface::INDEX_ACTION_REMOVE];
            if ($remove instanceof Zend_Search_Lucene_Document) {
              $self->_removeDocument($remove);
            }
          }
        }
        break;

      case Custom_Db_Table_Row_Observer::OBSERVE_PRE_DELETE:
        $documents = $row->getDocuments();

        foreach ($documents as $document) {
          $remove = reset($document);
          if ($remove instanceof Zend_Search_Lucene_Document) {
            $self->_removeDocument($remove);
          }
        }
        break;
    }

    return true;
  }

  /**
   * Reindexuje záznamy řádku
   *
   * @param Zend_Db_Table_Row_Abstract $row
   * @return boolean
   */
  private function _recordRow($row) {
    $result = true;

    $all = $row->getTable()->fetchAll();
    foreach ($all as $row) {
      $result = $result && self::record(Custom_Db_Table_Row_Observer::OBSERVE_POST_UPDATE, $row);
    }

    return $result;
  }

}
