<?php

/**
 * Třída pro práci s překlady textů
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_Model_Texts extends Zend_Db_Table_Abstract {

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_texts';

  /**
   * Sloupec primárního klíče
   *
   * @var string
   */
  protected $_primary = 'id';

  /**
   * Třída řádku
   *
   * @var string
   */
  protected $_rowClass = 'System_Model_Text';

  /**
   * Vrátí objekt překladu dle $textId
   *
   * @param string $id
   * @return System_Model_Text
   */
  public function getId($id) {
    if (!is_string($id)) {
      return null;
    }
    return $this->find($this->_sanitizeText($id))->current();
  }

  /**
   * Otestuje, zdali existuje objekt překladu
   *
   * @param string $id
   * @return boolean
   */
  public function hasId($id) {
    return is_object($this->getId($id));
  }

  /**
   * Vloží nový překlad do databáze
   *
   * @param string $id
   * @return System_Model_Text
   */
  public function addId($id) {
    $row = $this->createRow();
    $row->id = $this->_sanitizeText($id);
    $row->cs = null;
    $row->save();
    return $row;
  }

  /**
   * Odebere překlad z databáze
   *
   * @param string $id
   * @return boolean
   */
  public function removeId($id) {
    if (is_object($row = $this->getId($id))) {
      return $row->delete();
    }
    return false;
  }

  /**
   * Přidá sloupec s překlady do systémové tabulky překladů
   *
   * @param string $language
   * @return void
   */
  public function addTranslates($language) {
    if (!$this->exists($language)) {
      $alter = "ALTER TABLE {$this->info(self::NAME)} ADD {$language} "
             . "MEDIUMTEXT CHARACTER SET utf8 DEFAULT NULL";
      $this->getAdapter()->query($alter);

      $update = "UPDATE {$this->info(self::NAME)} SET {$language} = cs";
      $this->getAdapter()->query($update);

      $this->getMetadataCache()->clean();
    }
  }

  /**
   * Vrátí seznam s překlady v daném jazyku
   *
   * @param string $language
   * @return void
   */
  public function getTranslates($language) {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = serialize(__METHOD__.$language);

    if (!is_object($cache) || (false === ($result = $cache->load($key)))) {
      $sql = "SELECT id, {$language} FROM {$this->info(self::NAME)}";
      $result = $this->getAdapter()->fetchPairs($sql);
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Otestuje, zdali jazyk $language v tabulce existuje
   *
   * @param string $language
   * @return boolean
   */
  public function exists($language) {
    $columns = $this->info(self::COLS);
    return in_array($language, $columns);
  }

  /**
   * Vrátí pole možných jazyků
   *
   * @todo Dodělat
   * @return array
   */
  public static function getLanguages() {
    if (is_array($languages = System_Model_Settings::getValue('languages', 'Webbia', null)) &&
        !empty($languages)) {
      return $languages;
    }
    return array('cs' => 'čeština');
  }

  /**
   * Vrátí název jazyka dle $language
   *
   * @return string
   */
  public static function getName($language) {
    $names = self::getLanguages();
    if (array_key_exists($language, $names)) {
      return $names[$language];
    }
    return 'čeština';
  }

  /**
   * Zkusí vyhledat upgrade jazyka dle názvu
   *
   * stdClass:
   *  + language
   *  + writing
   *  + country
   *
   * @param string $language
   * @return stdClass|null
   */
  public static function search($language) {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = md5(__METHOD__.$language);

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $result = null;
      $upgrade = Zend_Locale::getTranslation($language, 'localeupgrade', $language);
      if (is_string($upgrade)) {
        $result = new stdClass();
        list($result->language, $result->writing, $result->country) = explode('_', $upgrade);
      }
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí vlajky dostupných mutací
   *
   * @param array $allowed
   * @return array
   */
  public static function fetchFlags(array $allowed = array(), $as_disabled = false) {
    $languages = self::getLanguages();
    foreach ($languages as $language => &$name) {
      $flag = $language;
      if ($as_disabled && !in_array($language, $allowed)) {
        $flag .= '_disabled';
      }
      $class = in_array($language, $allowed) ? '' : 'not-set';
      $name = array(
        'flag'  => $flag,
        'name'  => $name,
        'class' => $class
      );
    }
    return $languages;
  }

  /**
   * Upraví text $id pro indexový sloupec
   *
   * @param string $id
   * @return string
   */
  protected function _sanitizeText($id) {
    if (iconv_strlen($id) > 255) {
      $id = iconv_substr($id, 0, 255);
    }
    return trim($id);
  }

}
