<?php

/**
 * Třída objektu uživatele
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_Model_User extends Zend_Db_Table_Row_Abstract {

  /**
   * Název třídy modelu databáze
   *
   * @var string
   */
  protected $_tableClass = 'System_Model_Users';

  /**
   * Nastaví hodnotu buňky řádku
   *
   * @param string $columnName
   * @param mixed $value
   * @return void
   */
  public function __set($columnName, $value) {
    if ('is_active' == $columnName) {
      $auth = Zend_Auth::getInstance();
      if ($auth->hasIdentity() && ($auth->getIdentity()->id_user == $this->id_user)) {
        return;
      }
    }
    parent::__set($columnName, $value);
    if (('otp_type' == $columnName) &&
        ($this->_data['otp_type'] != $this->_cleanData['otp_type'])) {
      $this->otp_seed = $this->hotp_counter = null;
    }
  }

  /**
   * Vrátí celé jméno uživatele
   *
   * @return string
   */
  public function getFullName() {
    $names = array();

    $names[] = $this->first_name;
    if (is_string($nick_name = $this->nick_name) && !empty($nick_name)) {
      $names[] = $nick_name;
    }
    $names[] = $this->last_name;

    return trim(implode(' ', $names));
  }

  /**
   * Nastaví uživatele jako aktivního
   *
   * @return mixed
   */
  public function setActive() {
    $this->is_active = (int) true;
    return $this->save();
  }

  /**
   * Nastaví uživatele jako neaktivního
   *
   * @return mixed
   */
  public function setInactive() {
    $this->is_active = (int) false;
    return $this->save();
  }

  /**
   * Skryje heslo
   *
   * @return System_Model_User
   */
  public function hidePassword() {
    $this->_data['user_password'] = null;
    $this->_cleanData['user_password'] = null;
    return $this;
  }

  /**
   * Připraví data uživatele k použití OTP
   *
   * @return string  seed v base32
   */
  public function prepareUse() {
    $otpSeed = System_Model_OTP::GeneratePassword(10, false);
    $result = System_Model_OTP::Base32Encode($otpSeed);

    $this->otp_seed = System_Model_OTP::UnsafeEncrypt($otpSeed, $this->email);
    $this->hotp_counter = System_Model_Users::OTP_TYPE_HOTP == $this->otp_type ? 0 : null;
    $this->save();

    return $result;
  }

  /**
   * Ověří jednorázové heslo
   *
   * @param string $otp
   * @return boolean
   */
  public function verifyOTP($otp) {
    $otpSeed = System_Model_OTP::UnsafeDecrypt($this->otp_seed, $this->email);

    $result = false;
    if (System_Model_Users::OTP_TYPE_HOTP == $this->otp_type) {
      $counter = $this->hotp_counter;
      if ($result = System_Model_OTP::VerifyHOTP($otp, $otpSeed, $counter)) {
        $this->hotp_counter = $counter + 1;
      }
    }
    if (System_Model_Users::OTP_TYPE_TOTP == $this->otp_type) {
      $result = System_Model_OTP::VerifyTOTP($otp, $otpSeed);
    }

    return $result;
  }

  /**
   * Umožní aplikovat změnu řádku před vložením.
   *
   * @return void
   */
  protected function _insert() {
    $this->id_user = null;
    $this->date_create = date('Y-m-d H:i:s');

    parent::_insert();
  }

  /**
   * Umožní aplikovat změnu řádku před uložením.
   *
   * @return void
   */
  protected function _update() {
    $this->date_change = date('Y-m-d H:i:s');

    parent::_update();
  }

  /**
   * Zabezpečení hesla
   *
   * @param string $password
   * @return string
   */
  public static function EncryptPassword($password) {
    return password_hash($password, PASSWORD_BCRYPT, array('cost' => 12));
  }

}
