<?php

/**
 * Třída pro práci s uživately v tabulce webbia_users
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_Model_Users extends Zend_Db_Table_Abstract {

  const TTL = 600; // 10 min.

  const OTP_TYPE_HOTP = 'hotp';
  const OTP_TYPE_TOTP = 'totp';

  /**
   * Druhy OTP
   *
   * @var array
   */
  public static $OTP_TYPES = array(
    self::OTP_TYPE_HOTP => 'System_Users_Type_HOTP',
    self::OTP_TYPE_TOTP => 'System_Users_Type_TOTP'
  );

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_users';

  /**
   * Sloupec primárního klíče
   *
   * @var string
   */
  protected $_primary = 'id_user';

  /**
   * Třída řádku
   *
   * @var string
   */
  protected $_rowClass = 'System_Model_User';

  /**
   * Vrátí všechny uživatele nebo jejich počet
   *
   * @param integer $limit
   * @param integer $offset
   * @param boolean $count
   * @return Zend_Db_Table_Rowset|integer
   */
  public function getAllUsers($limit = null, $offset = null, $count = false) {
    $name = $this->info(self::NAME);
    if ($count) {
      $sql = "SELECT COUNT(*) FROM {$name}";
      return (int) $this->getAdapter()->fetchOne($sql);
    }
    return $this->fetchAll(null, 'last_name ASC', $limit, $offset);
  }

  /**
   * Vrátí uživatele dle $id_user
   *
   * @param integer $id_user
   * @return System_Model_User
   */
  public function getUserById($id_user) {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = md5(__METHOD__.$id_user);

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $result = $this->find($id_user)->current();
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí uživatele dle $email
   *
   * @param string $email
   * @param System_Model_User $ignoreUser
   * @return System_Model_User
   */
  public function getUserByEmail($email, System_Model_User $ignoreUser = null) {
    $select = $this->select()->where('email = ?', $email);
    if ($ignoreUser instanceof System_Model_User) {
      $select->where('email != ?', $ignoreUser->email);
    }
    return $this->fetchRow($select);
  }

  /**
   * Otestuje, zdali existuje uživatel s danými přihlašovacími údaji
   *
   * @param string $user_name
   * @param string $user_password
   * @param array $where
   * @return System_Model_User
   */
  public function userExists($user_name, $user_password, array $where = array()) {
    $where['user_name = ?'] = $user_name;
    $users = $this->fetchAll($where);
    foreach ($users as $user) {
      if (password_verify($user_password, $user->user_password)) {
        return $user;
      }
    }
    return null;
  }

  /**
   * Nastaví stav uživatelům
   *
   * @param array $id_users
   * @param integer $state
   * @return void
   */
  public function setUsersState(array $id_users, $state) {
    $auth = Zend_Auth::getInstance();
    if ($auth->hasIdentity() && in_array($id_user = $auth->getIdentity()->id_user, $id_users)) {
      unset($id_users[array_search($id_user, $id_users)]);
    }
    if (!empty($id_users)) {
      $this->update(array('is_active' => (int) $state), array('id_user IN (?)' => $id_users));
    }
  }

  /**
   * Zaznamená osobu a čas poslední operace přihlášeného uživatele
   *
   * @return void
   */
  public static function notify() {
    $self = new self();

    $ttl = self::TTL;
    $name = $self->info(self::NAME);
    $auth = Zend_Auth::getInstance();

    if (is_numeric($id_user = $auth->hasIdentity() ? $auth->getIdentity()->id_user : null)) {
      $sql = "UPDATE {$name} SET date_notify = NOW() WHERE id_user = {$id_user}";
      $self->getAdapter()->query($sql);
    }

    $sql = "UPDATE {$name} SET date_notify = NULL WHERE date_notify < (NOW() - INTERVAL {$ttl} SECOND)";
    $self->getAdapter()->query($sql);
  }

  /**
   * Vrátí seznam všech zalogovaných uživatelů
   *
   * @return Zend_Db_Table_Rowset
   */
  public static function getLoggedUsers() {
    $self = new self();
    return $self->fetchAll(array('date_notify IS NOT NULL'));
  }

  /**
   * Vrátí seznam všech aktivních uživatelů
   *
   * @return Zend_Db_Table_Rowset
   */
  public static function getActiveUsers() {
    $self = new self();
    return $self->fetchAll(array('is_active = ?' => (int) true));
  }

}
