// Požadovaný jazykový balíček
tinyMCEPopup.requireLangPack('columns');

/**
 * Dialog pro sloupce
 *
 * @type Object
 */
var ColumnsDialog = {

  /**
   * Minimum a maximum sloupců
   *
   * @type Object
   */
  limits: {min: 2, max: 6},

  /**
   * Šířky sloupců v procentech
   *
   * @type Array
   */
  widths: [50, 50],

  /**
   * Vnitřní obsahy sloupců
   *
   * @type Array
   */
  inners: [],

  /**
   * Počítadlo instancí bloků se sloupci na stránce
   *
   * @type Number
   */
  counter: 0,

  /**
   * Vrátí buňku tabulky pro nastavení šířky sloupce
   *
   * @param {Number} index
   * @param {Number} width
   * @returns {String}
   */
  _tableCell: function(index, width) {
    var
      id = 'width-' + index,
      value = isNaN(width) ? (100 / ColumnsDialog.limits.max).toFixed(2) : width.toFixed(2),
      output = '<td>\n'
             + '  <label for="' + id + '">\n'
             + '    ' + tinyMCEPopup.getLang('columns_dlg.width') + '\n'
             + '    <input type="text" name="' + id + '" id="' + id + '" value="' + value + '" size="4" />\n'
             + '    %\n'
             + '  </label>\n'
             + '</td>\n'
    ;
    return output;
  },

  /**
   * Vrátí šablonku sloupce
   *
   * @returns {String}
   */
  _template: function() {
    var
      columns = '',
      id = 'columns' + ColumnsDialog.counter
    ;
    for (var i = 0, j = ColumnsDialog.widths.length; i < j; i++) {
      var
        ID = id + '_column' + i,
        style = 'width: ' + (ColumnsDialog.widths[i]).toFixed(2) + '%',
        inner = ColumnsDialog.inners[i] ? ColumnsDialog.inners[i] : '<p>Lorem ipsum ' + (i + 1) + '</p>',
        column = '<div id="' + ID + '" class="columns-column" contenteditable="true" '
               + 'style="' + style + '">\n' + inner + '</div>\n'
      ;
      columns += column;
    }
    return columns;
  },

  /**
   * Vrátí obalující prvek sloupců
   *
   * @param {Node} element
   * @returns {Node}
   */
  _getWrapper: function(element) {
    while (element) {
      if (element.id && element.id.match(/^columns(\d+)$/)) {
        return element;
      }
      element = element.parentNode;
    }
    return null;
  },

  /**
   * Inicializace dialogu
   *
   * @returns {Void}
   */
  init: function() {

    /**
     * Vrátí shodu vzoru pattern s ID elementu
     *
     * @param {Node} element
     * @param {Regexp} pattern
     * @returns {Array}
     */
    function _elementMatch(element, pattern) {
      var match = null;
      while (null !== element) {
        if (element.id && (match = element.id.match(pattern))) {
          return match;
        }
        element = element.parentNode;
      }
      return null;
    }

    /**
     * Vrátí vnitřní obsah elementů
     *
     * @param {Number} count
     * @param {Number} index
     * @returns {Array}
     */
    function _getInners(count, index) {
      var result = [], id = 'columns' + index + '_column';
      for (var i = 0, node = null; i < count; i++) {
        if (node = tinyMCEPopup.editor.dom.get(id + i)) {
          result[i] = node.innerHTML;
        }
      }
      return result;
    }

    var
      node = tinyMCEPopup.editor.selection.getNode(),
      match = _elementMatch(node, /^columns(\d+)/),
      wrapper = match ? ColumnsDialog._getWrapper(node) : null,
      widths = wrapper ? wrapper.getAttribute('data-sizes') : null
    ;

    if (wrapper && match && widths) {
      ColumnsDialog.widths = widths.split('-').map(function(value) {
        return +value;
      });
    }

    if (match) {
      ColumnsDialog.counter = match[1];
      ColumnsDialog.inners = _getInners(ColumnsDialog.widths.length, ColumnsDialog.counter);
    } else {
      while (tinyMCEPopup.editor.dom.get('columns' + ColumnsDialog.counter)) {
        ColumnsDialog.counter++;
      }
    }

    var count = document.getElementById('count');
    count.setAttribute('min', ColumnsDialog.limits.min);
    count.setAttribute('max', ColumnsDialog.limits.max);
    count.value = ColumnsDialog.widths.length;

    ColumnsDialog.update();
  },

  /**
   * Reakce na změnu počtu sloupců
   *
   * @returns {Void}
   */
  change: function() {
    var count = +(document.getElementById('count').value);
    if (isNaN(count)) {
      return alert(tinyMCEPopup.getLang('columns_dlg.notNumber'));
    }
    if ((ColumnsDialog.limits.min > count) || (ColumnsDialog.limits.max < count)) {
      var error = tinyMCEPopup.getLang('columns_dlg.notInRange');
      return alert(error.replace('%min%', ColumnsDialog.limits.min).replace('%max%', ColumnsDialog.limits.max));
    }

    for (var i = 0, j = ColumnsDialog.widths.length; i < j; i++) {
      var element = document.getElementById('width-' + i), width = +(element.value);
      if (isNaN(width)) {
        element.style.outline = '2px solid red';
        return alert(tinyMCEPopup.getLang('columns_dlg.notPercent'));
      }
      ColumnsDialog.widths[i] = width;
    }

    if (count > ColumnsDialog.widths.length) {
      var width = (100 / ColumnsDialog.limits.max);
      while (count-- > ColumnsDialog.widths.length) {
        ColumnsDialog.widths[count] = width;
      }
    } else {
      ColumnsDialog.widths = ColumnsDialog.widths.slice(0, count);
    }

    ColumnsDialog.update();
  },

  /**
   * Změní obsah řádku tabulky pro změnu šířky sloupců
   *
   * @returns {Void}
   */
  update: function() {
    var wrapper = document.getElementById('widths-wrapper');

    wrapper.innerHTML = '';
    for (var i = 0, j = ColumnsDialog.widths.length; i < j; i++) {
      wrapper.innerHTML += ColumnsDialog._tableCell(i, ColumnsDialog.widths[i]);
    }

    ColumnsDialog.resize();
  },

  /**
   * Přepočítá procenta šířky sloupců
   *
   * @returns {Void}
   */
  calculate: function() {
    var width = (100 / ColumnsDialog.widths.length);
    for (var i = 0, j = ColumnsDialog.widths.length; i < j; i++) {
      ColumnsDialog.widths[i] = width;
    }
    ColumnsDialog.update();
  },

  /**
   * Po změně počtu sloupců změní velikost dialogového okna
   *
   * @returns {Void}
   */
  resize: function() {
    var
      self = tinyMCEPopup,
      wm = self.editor.windowManager,
      width = wm.features.width + 110 * (ColumnsDialog.widths.length - 2),
      win = parent.document.getElementById(self.id)
    ;
    wm.params.mce_width = width;
    tinyMCEPopup.resizeToInnerSize();
    win.style.left = (wm.features.left - (width - wm.features.width) / 2) + 'px';
  },

  /**
   * Spočítá celkovou šířku wrapperu
   *
   * @returns {Number}
   */
  sum: function() {
    var percent = 0;
    for (var i = 0, j = ColumnsDialog.widths.length; i < j; i++) {
      percent += ColumnsDialog.widths[i];
    }
    return percent;
  },

  /**
   * Zkontroluje celkovou šířku wrapperu
   *
   * @returns {Boolean}
   */
  check: function() {
    var percent = ColumnsDialog.sum();
    if (100.5 < percent) {
      return confirm(tinyMCEPopup.getLang('columns_dlg.more100'));
    }
    if (49.5 > percent) {
      return confirm(tinyMCEPopup.getLang('columns_dlg.less50'));
    }
    return true;
  },

  /**
   * Vloží HTML kód sloupců do editoru
   *
   * @returns {Void}
   */
  insert: function() {
    ColumnsDialog.change();
    if (!ColumnsDialog.check()) {
      return;
    }

    var
      percent = ColumnsDialog.sum(),
      template = ColumnsDialog._template(),
      node = tinyMCEPopup.editor.selection.getNode(),
      parent = ColumnsDialog._getWrapper(node),
      sizes = ColumnsDialog.widths.map(function(width) {return width.toFixed(2);}).join('-'),
      attributes = {
        class: 'columns-wrapper',
        contenteditable: 'false',
        'data-sizes': sizes,
        id: 'columns' + ColumnsDialog.counter,
        style: 'width: ' + percent + '%;'
      },
      parentId,
      element
    ;

    if (parent && (parentId = parent.id.match(/^columns(\d+)$/))) {
      element = tinyMCEPopup.editor.dom.get(parentId[0]);
      tinyMCEPopup.editor.dom.setAttribs(element, attributes);
      tinyMCEPopup.editor.dom.setHTML(element, template);
    } else {
      element = tinyMCEPopup.editor.dom.create('div', attributes, template);

      parent = node;
      if (!tinyMCEPopup.editor.dom.isBlock(node)) {
        parent = tinyMCEPopup.editor.dom.getParent(node, 'div,p,address,h3,h4,h5,h6,pre');
      }

      if (node.tagName && node.tagName === 'BODY') {
        tinyMCEPopup.editor.execCommand('mceInsertContent', false, tinyMCEPopup.editor.dom.getOuterHTML(element));
      } else {
        tinyMCEPopup.editor.dom.insertAfter(element, parent);
      }
    }

    tinyMCEPopup.close();
  }
};

// Inicializace tinyMCEPopupu - dialogu pro sloupce
tinyMCEPopup.onInit.add(ColumnsDialog.init, ColumnsDialog);
